#!/usr/bin/env python
# -*- coding: utf-8 -*-

import datetime
import random
import unittest

from mock import patch
from mock import MagicMock as Mock

from pyrax.cloudloadbalancers import CloudLoadBalancerClient
from pyrax.cloudloadbalancers import CloudLoadBalancer
from pyrax.cloudloadbalancers import Node
from pyrax.cloudloadbalancers import VirtualIP
from pyrax.cloudloadbalancers import _get_id
from pyrax.cloudloadbalancers import _time_string
from pyrax.cloudloadbalancers import assure_parent
from pyrax.cloudloadbalancers import assure_loadbalancer
import pyrax.exceptions as exc
import pyrax.utils as utils

from tests.unit import fakes

example_uri = "http://example.com"


class CloudLoadBalancerTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(CloudLoadBalancerTest, self).__init__(*args, **kwargs)

    def setUp(self):
        self.loadbalancer = fakes.FakeLoadBalancer()
        self.client = fakes.FakeLoadBalancerClient()

    def tearDown(self):
        self.loadbalancer = None
        self.client = None

    def test_get_id(self):
        target = "test_id"
        class Obj_with_id(object):
            id = target
        obj = Obj_with_id()
        self.assertEqual(_get_id(obj), target)
        self.assertEqual(_get_id(obj), target)
        self.assertEqual(_get_id(obj.id), target)

    def test_assure_parent_fail(self):
        orphan_node = Node(address="fake", port="fake")
        self.assertRaises(exc.UnattachedNode, orphan_node.update)

    def test_assure_parent_succeed(self):
        adopted_node = Node(address="fake", port="fake", parent=fakes.FakeLoadBalancer())
        diff = "DIFF"
        adopted_node._diff = Mock(return_value=diff)
        adopted_node.parent.update_node = Mock()
        adopted_node.update()
        adopted_node.parent.update_node.assert_called_once_with(adopted_node, diff)
        
    def test_assure_loadbalancer(self):
        class TestClient(object):
            _manager = fakes.FakeManager()
            
            @assure_loadbalancer
            def test_method(self, loadbalancer):
                return loadbalancer

        client = TestClient()
        client._manager.get = Mock(return_value=self.loadbalancer)
        # Pass the loadbalancer
        ret = client.test_method(self.loadbalancer)
        self.assertTrue(ret is self.loadbalancer)
        # Pass the ID
        ret = client.test_method(self.loadbalancer.id)
        self.assertTrue(ret is self.loadbalancer)

    def test_time_string_empty(self):
        testval = None
        self.assertEqual(_time_string(testval), "")

    def test_time_string_invalid(self):
        testval = "abcde"
        self.assertRaises(exc.InvalidDateTimeString, _time_string, testval)

    def test_time_string_date(self):
        dt = "1999-12-31"
        self.assertEqual(_time_string(dt), "1999-12-31T00:00:00")

    def test_time_string_datetime(self):
        dt = "1999-12-31 23:59:59"
        self.assertEqual(_time_string(dt), "1999-12-31T23:59:59")

    def test_add_nodes_client(self):
        clt = self.client
        lb = self.loadbalancer
        nd = fakes.FakeNode()
        lb.manager.add_nodes = Mock()
        clt.add_nodes(lb, nd)
        lb.manager.add_nodes.assert_called_once_with(lb, nd)

    def test_add_virtualip_client(self):
        clt = self.client
        lb = self.loadbalancer
        vip = fakes.FakeVirtualIP()
        lb.manager.add_virtualip = Mock()
        clt.add_virtualip(lb, vip)
        lb.manager.add_virtualip.assert_called_once_with(lb, vip)

    def test_add_details_nodes(self):
        fake_node_info = {"address": "0.0.0.0", "id": 1, "type": "PRIMARY", "port":80,
                "status": "OFFLINE", "condition": "ENABLED"}
        info = {"nodes": [fake_node_info]}
        lb = fakes.FakeLoadBalancer(name="fake", info=info)
        node = lb.nodes[0]
        self.assertEqual(node.address, fake_node_info["address"])

    def test_add_details_virtualips(self):
        fake_vip_info = {"address": "0.0.0.0", "id": 1, "type": "PUBLIC", "ipVersion": "IPV4"}
        info = {"virtualIps": [fake_vip_info]}
        lb = fakes.FakeLoadBalancer(name="fake", info=info)
        vip = lb.virtual_ips[0]
        self.assertEqual(vip.address, fake_vip_info["address"])

    def test_add_details_session_persistence(self):
        info = {"sessionPersistence": {"persistenceType": "fake"}}
        lb = fakes.FakeLoadBalancer(name="fake", info=info)
        self.assertEqual(lb.sessionPersistence, info["sessionPersistence"]["persistenceType"])

    def test_add_details_cluster(self):
        info = {"cluster": {"name": "fake"}}
        lb = fakes.FakeLoadBalancer(name="fake", info=info)
        self.assertEqual(lb.cluster, info["cluster"]["name"])

    def test_client_delete_node(self):
        clt = self.client
        lb = self.loadbalancer
        nd = fakes.FakeNode()
        nd.parent = lb
        lb.manager.delete_node = Mock()
        clt.delete_node(nd)
        lb.manager.delete_node.assert_called_once_with(lb, nd)

    def test_client_update_node(self):
        clt = self.client
        lb = self.loadbalancer
        nd = fakes.FakeNode()
        nd.parent = lb
        diff = "DIFF"
        nd._diff = Mock(return_value=diff)
        lb.manager.update_node = Mock()
        clt.update_node(nd)
        lb.manager.update_node.assert_called_once_with(nd, diff=diff)

    def test_client_delete_virtualip(self):
        clt = self.client
        lb = self.loadbalancer
        vip = fakes.FakeVirtualIP()
        vip.parent = lb
        lb.manager.delete_virtualip = Mock()
        clt.delete_virtualip(vip)
        lb.manager.delete_virtualip.assert_called_once_with(lb, vip)

    def test_client_get_access_list(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.get_access_list = Mock()
        clt.get_access_list(lb)
        lb.manager.get_access_list.assert_called_once_with(lb)

    def test_client_add_access_list(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.add_access_list = Mock()
        alist = {"fake": "fake"}
        clt.add_access_list(lb, alist)
        lb.manager.add_access_list.assert_called_once_with(lb, alist)

    def test_client_delete_access_list(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.delete_access_list = Mock()
        clt.delete_access_list(lb)
        lb.manager.delete_access_list.assert_called_once_with(lb)

    def test_client_delete_access_list_items(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.delete_access_list_items = Mock()
        fake_ids = [1, 2, 3]
        clt.delete_access_list_items(lb, fake_ids)
        lb.manager.delete_access_list_items.assert_called_once_with(lb, fake_ids)

    def test_client_get_health_monitor(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.get_health_monitor = Mock()
        clt.get_health_monitor(lb)
        lb.manager.get_health_monitor.assert_called_once_with(lb)

    def test_client_add_health_monitor(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.add_health_monitor = Mock()
        fake_type = "fake"
        fake_delay = 99
        fake_timeout = 99
        fake_attemptsBeforeDeactivation = 99
        fake_path = "/fake"
        fake_statusRegex = ".*fake.*"
        fake_bodyRegex = ".*fake.*"
        fake_hostHeader = "fake"
        clt.add_health_monitor(lb, type=fake_type, delay=fake_delay, timeout=fake_timeout,
                attemptsBeforeDeactivation=fake_attemptsBeforeDeactivation,
                path=fake_path, statusRegex=fake_statusRegex, bodyRegex=fake_bodyRegex,
                hostHeader=fake_hostHeader)
        lb.manager.add_health_monitor.assert_called_once_with(lb, type=fake_type,
                delay=fake_delay, timeout=fake_timeout,
                attemptsBeforeDeactivation=fake_attemptsBeforeDeactivation,
                path=fake_path, statusRegex=fake_statusRegex, bodyRegex=fake_bodyRegex,
                hostHeader=fake_hostHeader)

    def test_client_delete_health_monitor(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.delete_health_monitor = Mock()
        clt.delete_health_monitor(lb)
        lb.manager.delete_health_monitor.assert_called_once_with(lb)

    def test_client_get_connection_throttle(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.get_connection_throttle = Mock()
        clt.get_connection_throttle(lb)
        lb.manager.get_connection_throttle.assert_called_once_with(lb)

    def test_client_add_connection_throttle(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.add_connection_throttle = Mock()
        fake_maxConnectionRate = 99
        fake_maxConnections = 99
        fake_minConnections = 99
        fake_rateInterval = 99
        clt.add_connection_throttle(lb)
        self.assertEqual(lb.manager.add_connection_throttle.call_count, 0)
        clt.add_connection_throttle(lb, maxConnectionRate=fake_maxConnectionRate,
                maxConnections=fake_minConnections, minConnections=fake_minConnections,
                rateInterval=fake_rateInterval)
        lb.manager.add_connection_throttle.assert_called_once_with(lb,
                maxConnectionRate=fake_maxConnectionRate,
                maxConnections=fake_minConnections, minConnections=fake_minConnections,
                rateInterval=fake_rateInterval)

    def test_client_delete_connection_throttle(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.delete_connection_throttle = Mock()
        clt.delete_connection_throttle(lb)
        lb.manager.delete_connection_throttle.assert_called_once_with(lb)

    def test_client_get_ssl_termination(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.get_ssl_termination = Mock()
        clt.get_ssl_termination(lb)
        lb.manager.get_ssl_termination.assert_called_once_with(lb)

    def test_client_add_ssl_termination(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.add_ssl_termination = Mock()
        fake_securePort = 99
        fake_privatekey = "fake"
        fake_certificate = "fake"
        fake_intermediateCertificate = "fake"
        fake_enabled = True
        fake_secureTrafficOnly = False
        clt.add_ssl_termination(lb, securePort=fake_securePort, privatekey=fake_privatekey,
                certificate=fake_certificate, intermediateCertificate=fake_intermediateCertificate,
                enabled=fake_enabled, secureTrafficOnly=fake_secureTrafficOnly)
        lb.manager.add_ssl_termination.assert_called_once_with(lb,
                securePort=fake_securePort, privatekey=fake_privatekey,
                certificate=fake_certificate, intermediateCertificate=fake_intermediateCertificate,
                enabled=fake_enabled, secureTrafficOnly=fake_secureTrafficOnly)

    def test_client_update_ssl_termination(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.update_ssl_termination = Mock()
        fake_securePort = 99
        fake_enabled = True
        fake_secureTrafficOnly = False
        clt.update_ssl_termination(lb, securePort=fake_securePort, enabled=fake_enabled,
                secureTrafficOnly=fake_secureTrafficOnly)
        lb.manager.update_ssl_termination.assert_called_once_with(lb,
                securePort=fake_securePort, enabled=fake_enabled,
                secureTrafficOnly=fake_secureTrafficOnly)

    def test_client_delete_ssl_termination(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.delete_ssl_termination = Mock()
        clt.delete_ssl_termination(lb)
        lb.manager.delete_ssl_termination.assert_called_once_with(lb)

    def test_client_get_metadata(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.get_metadata = Mock()
        clt.get_metadata(lb)
        lb.manager.get_metadata.assert_called_once_with(lb)

    def test_client_set_metadata(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.set_metadata = Mock()
        meta = {"fake": "fake"}
        clt.set_metadata(lb, meta)
        lb.manager.set_metadata.assert_called_once_with(lb, meta)

    def test_client_update_metadata(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.update_metadata = Mock()
        meta = {"fake": "fake"}
        clt.update_metadata(lb, meta)
        lb.manager.update_metadata.assert_called_once_with(lb, meta)

    def test_client_delete_metadata(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.delete_metadata = Mock()
        keys = [1, 2, 3]
        clt.delete_metadata(lb, keys=keys)
        lb.manager.delete_metadata.assert_called_once_with(lb, keys=keys)

    def test_client_get_metadata_for_node(self):
        clt = self.client
        lb = self.loadbalancer
        nd = fakes.FakeNode()
        lb.manager.get_metadata = Mock()
        clt.get_metadata_for_node(lb, nd)
        lb.manager.get_metadata.assert_called_once_with(lb, node=nd)

    def test_client_set_metadata_for_node(self):
        clt = self.client
        lb = self.loadbalancer
        nd = fakes.FakeNode()
        lb.manager.set_metadata = Mock()
        meta = {"fake": "fake"}
        clt.set_metadata_for_node(lb, nd, meta)
        lb.manager.set_metadata.assert_called_once_with(lb, meta, node=nd)

    def test_client_update_metadata_for_node(self):
        clt = self.client
        lb = self.loadbalancer
        nd = fakes.FakeNode()
        lb.manager.update_metadata = Mock()
        meta = {"fake": "fake"}
        clt.update_metadata_for_node(lb, nd, meta)
        lb.manager.update_metadata.assert_called_once_with(lb, meta, node=nd)

    def test_client_delete_metadata_for_node(self):
        clt = self.client
        lb = self.loadbalancer
        nd = fakes.FakeNode()
        lb.manager.delete_metadata = Mock()
        keys = [1, 2, 3]
        clt.delete_metadata_for_node(lb, nd, keys=keys)
        lb.manager.delete_metadata.assert_called_once_with(lb, node=nd, keys=keys)

    def test_client_get_error_page(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.get_error_page = Mock()
        clt.get_error_page(lb)
        lb.manager.get_error_page.assert_called_once_with(lb)

    def test_client_set_error_page(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.set_error_page = Mock()
        ep = "<fake>"
        clt.set_error_page(lb, ep)
        lb.manager.set_error_page.assert_called_once_with(lb, ep)

    def test_client_clear_error_page(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.clear_error_page = Mock()
        clt.clear_error_page(lb)
        lb.manager.clear_error_page.assert_called_once_with(lb)

    def test_client_get_connection_logging(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.get_connection_logging = Mock()
        clt.get_connection_logging(lb)
        lb.manager.get_connection_logging.assert_called_once_with(lb)

    def test_client_set_connection_logging(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.set_connection_logging = Mock()
        clt.set_connection_logging(lb, True)
        lb.manager.set_connection_logging.assert_called_once_with(lb, True)

    def test_client_get_content_caching(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.get_content_caching = Mock()
        clt.get_content_caching(lb)
        lb.manager.get_content_caching.assert_called_once_with(lb)

    def test_client_set_content_caching(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.set_content_caching = Mock()
        clt.set_content_caching(lb, True)
        lb.manager.set_content_caching.assert_called_once_with(lb, True)

    def test_client_get_session_persistence(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.get_session_persistence = Mock()
        clt.get_session_persistence(lb)
        lb.manager.get_session_persistence.assert_called_once_with(lb)

    def test_client_set_session_persistence_bad(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.set_session_persistence = Mock()
        self.assertRaises(exc.InvalidSessionPersistenceType, clt.set_session_persistence, lb, "BAD")

    def test_client_set_session_persistence(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.set_session_persistence = Mock()
        clt.set_session_persistence(lb, "HTTP_COOKIE")
        lb.manager.set_session_persistence.assert_called_once_with(lb, "HTTP_COOKIE")

    def test_client_clear_session_persistence(self):
        clt = self.client
        lb = self.loadbalancer
        lb.manager.delete_session_persistence = Mock()
        clt.set_session_persistence(lb, "")
        lb.manager.delete_session_persistence.assert_called_once_with(lb)

    def test_mgr_add_nodes(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_post = Mock(return_value=({}, {}))
        nd = fakes.FakeNode()
        mgr.add_nodes(lb, nd)
        uri = "/loadbalancers/%s/nodes" % lb.id
        ndict = nd.to_dict()
        mgr.api.method_post.assert_called_once_with(uri, body={"nodes": [ndict]})

    def test_mgr_delete_node(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_delete = Mock(return_value=({}, {}))
        nd = fakes.FakeNode()
        nd.parent = lb
        uri = "/loadbalancers/%s/nodes/%s" % (lb.id, nd.id)
        mgr.delete_node(lb, nd)
        mgr.api.method_delete.assert_called_once_with(uri)

    def test_mgr_delete_unattached_node(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_delete = Mock(return_value=({}, {}))
        nd = fakes.FakeNode()
        self.assertRaises(exc.UnattachedNode, mgr.delete_node, lb, nd)

    def test_mgr_update_node(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        nd = fakes.FakeNode()
        nd.parent = lb
        uri = "/loadbalancers/%s/nodes/%s" % (lb.id, nd.id)
        mgr.update_node(nd)
        mgr.api.method_put.assert_called_once_with(uri, body={"node": {}})

    def test_mgr_update_unattached_node(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        nd = fakes.FakeNode()
        self.assertRaises(exc.UnattachedNode, mgr.update_node, nd)

    def test_mgr_add_virtualip(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_post = Mock(return_value=({}, {}))
        vip = fakes.FakeVirtualIP()
        mgr.add_virtualip(lb, vip)
        uri = "/loadbalancers/%s/virtualips" % lb.id
        mgr.api.method_post.assert_called_once_with(uri, body=vip.to_dict())

    def test_mgr_delete_virtualip(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_delete = Mock(return_value=({}, {}))
        vip = fakes.FakeVirtualIP()
        vip.parent = lb
        uri = "/loadbalancers/%s/virtualips/%s" % (lb.id, vip.id)
        mgr.delete_virtualip(lb, vip)
        mgr.api.method_delete.assert_called_once_with(uri)

    def test_mgr_delete_unattached_virtualip(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        vip = fakes.FakeVirtualIP()
        self.assertRaises(exc.UnattachedVirtualIP, mgr.delete_virtualip, lb, vip)

    def test_mgr_get_access_list(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(return_value=({}, {}))
        mgr.get_access_list(lb)
        uri = "/loadbalancers/%s/accesslist" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_add_access_list(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_post = Mock(return_value=({}, {}))
        alist = {"fake": "fake"}
        mgr.add_access_list(lb, alist)
        uri = "/loadbalancers/%s/accesslist" % lb.id
        req_body = {"accessList": alist}
        mgr.api.method_post.assert_called_once_with(uri, body=req_body)

    def test_mgr_delete_access_list(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_delete = Mock(return_value=({}, {}))
        uri = "/loadbalancers/%s/accesslist" % lb.id
        mgr.delete_access_list(lb)
        mgr.api.method_delete.assert_called_once_with(uri)

    def test_mgr_delete_access_list_items(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_delete = Mock(return_value=({}, {}))
        fake_id = 3
        acc_ids = [{"id": 1}, {"id": 2}, {"id": 3}]
        mgr.get_access_list = Mock(return_value=acc_ids)
        uri = "/loadbalancers/%s/accesslist?id=3" % lb.id
        mgr.delete_access_list_items(lb, fake_id)
        mgr.api.method_delete.assert_called_once_with(uri)

    def test_mgr_delete_access_list_items_invalid(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_delete = Mock(return_value=({}, {}))
        ids = [1, 2, 99]
        acc_ids = [{"id": 1}, {"id": 2}, {"id": 3}]
        mgr.get_access_list = Mock(return_value=acc_ids)
        self.assertRaises(exc.AccessListIDNotFound, mgr.delete_access_list_items,
                lb, ids)

    def test_mgr_get_health_monitor(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(return_value=({}, {}))
        mgr.get_health_monitor(lb)
        uri = "/loadbalancers/%s/healthmonitor" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_add_health_monitor(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        fake_type = "HTTP"
        fake_delay = 99
        fake_timeout = 99
        fake_attemptsBeforeDeactivation = 99
        fake_path = "/fake"
        fake_statusRegex = ".*fake.*"
        fake_bodyRegex = ".*fake.*"
        fake_hostHeader = "fake"
        lb.protocol = fake_type
        mgr.add_health_monitor(lb, type=fake_type, delay=fake_delay, timeout=fake_timeout,
                attemptsBeforeDeactivation=fake_attemptsBeforeDeactivation,
                path=fake_path, statusRegex=fake_statusRegex, bodyRegex=fake_bodyRegex,
                hostHeader=fake_hostHeader)
        uri = "/loadbalancers/%s/healthmonitor" % lb.id
        req_body = {"healthMonitor": {
                "type": fake_type,
                "delay": fake_delay,
                "timeout": fake_timeout,
                "attemptsBeforeDeactivation": fake_attemptsBeforeDeactivation,
                "path": fake_path,
                "statusRegex": fake_statusRegex,
                "bodyRegex": fake_bodyRegex,
                "hostHeader": fake_hostHeader,
                }}
        mgr.api.method_put.assert_called_once_with(uri, body=req_body)

    def test_mgr_add_health_monitor_bad_protocol(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        fake_type = "HTTP"
        fake_delay = 99
        fake_timeout = 99
        fake_attemptsBeforeDeactivation = 99
        fake_path = "/fake"
        fake_statusRegex = ".*fake.*"
        fake_bodyRegex = ".*fake.*"
        fake_hostHeader = "fake"
        lb.protocol = "BAD"
        self.assertRaises(exc.ProtocolMismatch, mgr.add_health_monitor, lb,
                type=fake_type, delay=fake_delay, timeout=fake_timeout,
                attemptsBeforeDeactivation=fake_attemptsBeforeDeactivation,
                path=fake_path, statusRegex=fake_statusRegex, bodyRegex=fake_bodyRegex,
                hostHeader=fake_hostHeader)

    def test_mgr_add_health_monitor_missing_cert(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        fake_type = "HTTP"
        fake_delay = 99
        fake_timeout = 99
        fake_attemptsBeforeDeactivation = 99
        fake_path = "/fake"
        fake_statusRegex = None
        fake_bodyRegex = None
        fake_hostHeader = "fake"
        lb.protocol = fake_type
        self.assertRaises(exc.MissingHealthMonitorSettings, mgr.add_health_monitor, lb,
                type=fake_type, delay=fake_delay, timeout=fake_timeout,
                attemptsBeforeDeactivation=fake_attemptsBeforeDeactivation,
                path=fake_path, statusRegex=fake_statusRegex, bodyRegex=fake_bodyRegex,
                hostHeader=fake_hostHeader)

    def test_mgr_delete_health_monitor(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_delete = Mock(return_value=({}, {}))
        uri = "/loadbalancers/%s/healthmonitor" % lb.id
        mgr.delete_health_monitor(lb)
        mgr.api.method_delete.assert_called_once_with(uri)

    def test_mgr_get_connection_throttle(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(return_value=({}, {}))
        mgr.get_connection_throttle(lb)
        uri = "/loadbalancers/%s/connectionthrottle" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_add_connection_throttle(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        fake_maxConnectionRate = 99
        fake_maxConnections = 99
        fake_minConnections = 99
        fake_rateInterval = 99
        mgr.add_connection_throttle(lb, maxConnectionRate=fake_maxConnectionRate,
                maxConnections=fake_minConnections, minConnections=fake_minConnections,
                rateInterval=fake_rateInterval)
        uri = "/loadbalancers/%s/connectionthrottle" % lb.id
        req_body = {"connectionThrottle": {
                "maxConnectionRate": fake_maxConnectionRate,
                "maxConnections": fake_minConnections,
                "minConnections": fake_minConnections,
                "rateInterval": fake_rateInterval,
                }}
        mgr.api.method_put.assert_called_once_with(uri, body=req_body)

    def test_mgr_delete_connection_throttle(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_delete = Mock(return_value=({}, {}))
        uri = "/loadbalancers/%s/connectionthrottle" % lb.id
        mgr.delete_connection_throttle(lb)
        mgr.api.method_delete.assert_called_once_with(uri)

    def test_mgr_get_ssl_termination(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(return_value=({}, {}))
        mgr.get_ssl_termination(lb)
        uri = "/loadbalancers/%s/ssltermination" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_get_ssl_termination_missing(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(side_effect=exc.NotFound("fake"))
        ret = mgr.get_ssl_termination(lb)
        self.assertEqual(ret, {})

    def test_mgr_add_ssl_termination(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        fake_securePort = 99
        fake_privatekey = "fake"
        fake_certificate = "fake"
        fake_intermediateCertificate = "fake"
        fake_enabled = True
        fake_secureTrafficOnly = False
        mgr.add_ssl_termination(lb, securePort=fake_securePort, privatekey=fake_privatekey,
                certificate=fake_certificate, intermediateCertificate=fake_intermediateCertificate,
                enabled=fake_enabled, secureTrafficOnly=fake_secureTrafficOnly)
        uri = "/loadbalancers/%s/ssltermination" % lb.id
        req_body = {"sslTermination": {
                "certificate": fake_certificate,
                "enabled": fake_enabled,
                "secureTrafficOnly": fake_secureTrafficOnly,
                "privatekey": fake_privatekey,
                "intermediateCertificate": fake_intermediateCertificate,
                "securePort": fake_securePort
                }}
        mgr.api.method_put.assert_called_once_with(uri, body=req_body)

    def test_mgr_update_ssl_termination_no_config(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.get_ssl_termination = Mock(return_value=None)
        self.assertRaises(exc.NoSSLTerminationConfiguration, mgr.update_ssl_termination,
                lb)

    def test_mgr_update_ssl_termination(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        fake_securePort = 99
        fake_enabled = True
        fake_secureTrafficOnly = False
        fake_config = {"securePort": fake_securePort,
                "enabled": fake_enabled,
                "secureTrafficOnly": fake_secureTrafficOnly}
        mgr.get_ssl_termination = Mock(return_value=fake_config)

        mgr.update_ssl_termination(lb)
        uri = "/loadbalancers/%s/ssltermination" % lb.id
        req_body = {"sslTermination": {
                "enabled": fake_enabled,
                "secureTrafficOnly": fake_secureTrafficOnly,
                "securePort": fake_securePort
                }}
        mgr.api.method_put.assert_called_once_with(uri, body=req_body)

    def test_mgr_delete_ssl_termination(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_delete = Mock(return_value=({}, {}))
        uri = "/loadbalancers/%s/ssltermination" % lb.id
        mgr.delete_ssl_termination(lb)
        mgr.api.method_delete.assert_called_once_with(uri)

    def test_mgr_get_metadata(self):
        lb = self.loadbalancer
        mgr = lb.manager
        fake_meta = {"fakekey": "fakeval"}
        fake_screwy_meta = [{"key": "fakekey", "value": "fakeval", "id": 1}]
        mgr.api.method_get = Mock(return_value=({}, {"metadata": fake_screwy_meta}))
        ret = mgr.get_metadata(lb)
        uri = "/loadbalancers/%s/metadata" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)
        self.assertEqual(ret, fake_meta)

    def test_mgr_get_node_metadata(self):
        lb = self.loadbalancer
        mgr = lb.manager
        fake_meta = {"fakekey": "fakeval"}
        fake_screwy_meta = [{"key": "fakekey", "value": "fakeval", "id": 1}]
        mgr.api.method_get = Mock(return_value=({}, {"metadata": fake_screwy_meta}))
        nd = fakes.FakeNode()
        ret = mgr.get_metadata(lb, node=nd)
        uri = "/loadbalancers/%s/nodes/%s/metadata" % (lb.id, nd.id)
        mgr.api.method_get.assert_called_once_with(uri)
        self.assertEqual(ret, fake_meta)

    def test_mgr_get_metadata_raw(self):
        lb = self.loadbalancer
        mgr = lb.manager
        fake_meta = {"fakekey": "fakeval"}
        fake_screwy_meta = [{"key": "fakekey", "value": "fakeval", "id": 1}]
        mgr.api.method_get = Mock(return_value=({}, {"metadata": fake_screwy_meta}))
        ret = mgr.get_metadata(lb, raw=True)
        uri = "/loadbalancers/%s/metadata" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)
        self.assertEqual(ret, fake_screwy_meta)

    def test_mgr_set_metadata(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.delete_metadata = Mock()
        fake_meta = {"fakekey": "fakeval"}
        fake_screwy_meta = [{"key": "fakekey", "value": "fakeval"}]
        mgr.api.method_post = Mock(return_value=({}, {"metadata": fake_screwy_meta}))
        mgr.set_metadata(lb, fake_meta)
        uri = "/loadbalancers/%s/metadata" % lb.id
        req_body = {"metadata": fake_screwy_meta}
        mgr.api.method_post.assert_called_once_with(uri, body=req_body)

    def test_mgr_set_node_metadata(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.delete_metadata = Mock()
        fake_meta = {"fakekey": "fakeval"}
        fake_screwy_meta = [{"key": "fakekey", "value": "fakeval"}]
        mgr.api.method_post = Mock(return_value=({}, {"metadata": fake_screwy_meta}))
        nd = fakes.FakeNode()
        mgr.set_metadata(lb, fake_meta, node=nd)
        uri = "/loadbalancers/%s/nodes/%s/metadata" % (lb.id, nd.id)
        req_body = {"metadata": fake_screwy_meta}
        mgr.api.method_post.assert_called_once_with(uri, body=req_body)

    def test_mgr_update_metadata(self):
        lb = self.loadbalancer
        mgr = lb.manager
        fake_meta = {"fakekey": "fakeval"}
        fake_screwy_meta = [{"key": "fakekey", "value": "fakeval", "id": 1}]
        fake_new_meta = {"fakekey": "updated", "fakeNEWkey": "fakeNEWval"}
        fake_screwy_upd_meta = [{"key": "fakekey", "value": "updated"}]
        fake_screwy_new_meta = [{"key": "fakeNEWkey", "value": "fakeNEWval"}]
        mgr.get_metadata = Mock(return_value=fake_screwy_meta)
        mgr.api.method_post = Mock(return_value=({}, {}))
        mgr.api.method_put = Mock(return_value=({}, {}))
        mgr.update_metadata(lb, fake_new_meta)
        uri = "/loadbalancers/%s/metadata" % lb.id
        req_body = {"metadata": fake_screwy_new_meta}
        upd_req_body = {"meta": {"value": "updated"}}
        mgr.api.method_post.assert_called_once_with(uri, body=req_body)
        mgr.api.method_put.assert_called_once_with(uri, body=upd_req_body)

    def test_mgr_update_node_metadata(self):
        lb = self.loadbalancer
        mgr = lb.manager
        nd = fakes.FakeNode()
        fake_meta = {"fakekey": "fakeval"}
        fake_screwy_meta = [{"key": "fakekey", "value": "fakeval", "id": 1}]
        fake_new_meta = {"fakekey": "updated", "fakeNEWkey": "fakeNEWval"}
        fake_screwy_upd_meta = [{"key": "fakekey", "value": "updated"}]
        fake_screwy_new_meta = [{"key": "fakeNEWkey", "value": "fakeNEWval"}]
        mgr.get_metadata = Mock(return_value=fake_screwy_meta)
        mgr.api.method_post = Mock(return_value=({}, {}))
        mgr.api.method_put = Mock(return_value=({}, {}))
        mgr.update_metadata(lb, fake_new_meta, node=nd)
        put_uri = "/loadbalancers/%s/nodes/%s/metadata/1" % (lb.id, nd.id)
        post_uri = "/loadbalancers/%s/nodes/%s/metadata" % (lb.id, nd.id)
        req_body = {"metadata": fake_screwy_new_meta}
        upd_req_body = {"meta": {"value": "updated"}}
        mgr.api.method_post.assert_called_once_with(post_uri, body=req_body)
        mgr.api.method_put.assert_called_once_with(put_uri, body=upd_req_body)

    def test_mgr_delete_metadata(self):
        lb = self.loadbalancer
        mgr = lb.manager
        fake_meta = {"fakekey": "fakeval"}
        fake_screwy_meta = [{"key": "fakekey", "value": "fakeval", "id": 1}]
        fake_meta_key = fake_screwy_meta[0]["key"]
        fake_meta_id = fake_screwy_meta[0]["id"]
        mgr.get_metadata = Mock(return_value=fake_screwy_meta)
        mgr.api.method_delete = Mock(return_value=({}, {}))
        # No match, should be a noop
        mgr.delete_metadata(lb, keys="BAD")
        self.assertEqual(mgr.api.method_delete.call_count, 0)
        mgr.delete_metadata(lb, keys=fake_meta_key)
        uri = "/loadbalancers/%s/metadata?id=%s" % (lb.id, fake_meta_id)
        mgr.api.method_delete.assert_called_once_with(uri)

    def test_mgr_delete_node_metadata(self):
        lb = self.loadbalancer
        mgr = lb.manager
        nd = fakes.FakeNode()
        fake_meta = {"fakekey": "fakeval"}
        fake_screwy_meta = [{"key": "fakekey", "value": "fakeval", "id": 1}]
        fake_meta_key = fake_screwy_meta[0]["key"]
        fake_meta_id = fake_screwy_meta[0]["id"]
        mgr.get_metadata = Mock(return_value=fake_screwy_meta)
        mgr.api.method_delete = Mock(return_value=({}, {}))
        # No match, should be a noop
        mgr.delete_metadata(lb, keys="BAD", node=nd)
        self.assertEqual(mgr.api.method_delete.call_count, 0)
        mgr.delete_metadata(lb, keys=fake_meta_key, node=nd)
        uri = "/loadbalancers/%s/nodes/%s/metadata?id=%s" % (lb.id, nd.id, fake_meta_id)
        mgr.api.method_delete.assert_called_once_with(uri)

    def test_mgr_get_error_page(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(return_value=({}, {}))
        mgr.get_error_page(lb)
        uri = "/loadbalancers/%s/errorpage" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_set_error_page(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        ep = "<fake>"
        mgr.set_error_page(lb, ep)
        uri = "/loadbalancers/%s/errorpage" % lb.id
        req_body = {"errorpage": {"content": ep}}
        mgr.api.method_put.assert_called_once_with(uri, body=req_body)

    def test_mgr_clear_error_page(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_delete = Mock(return_value=({}, {}))
        uri = "/loadbalancers/%s/errorpage" % lb.id
        mgr.clear_error_page(lb)
        mgr.api.method_delete.assert_called_once_with(uri)

    def test_mgr_get_usage_all(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(return_value=({}, {}))
        mgr.get_usage()
        uri = "/loadbalancers/usage"
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_get_usage_for_lb(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(return_value=({}, {}))
        mgr.get_usage(loadbalancer=lb)
        uri = "/loadbalancers/%s/usage" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_get_usage_period(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(return_value=({}, {}))
        start = "1999-12-31"
        end = "2000-01-01"
        start_iso = "1999-12-31T00:00:00"
        end_iso = "2000-01-01T00:00:00"
        mgr.get_usage(start=start, end=end)
        uri = "/loadbalancers/usage?startTime=%s&endTime=%s" % (start_iso, end_iso)
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_get_stats(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(return_value=({}, {}))
        mgr.get_stats(lb)
        uri = "/loadbalancers/%s/stats" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_get_session_persistence(self):
        lb = self.loadbalancer
        mgr = lb.manager
        fake_type = "FAKE"
        body = {"sessionPersistence": {"persistenceType": fake_type}}
        mgr.api.method_get = Mock(return_value=({}, body))
        mgr.get_session_persistence(lb)
        uri = "/loadbalancers/%s/sessionpersistence" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_set_session_persistence(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        fake_type = "FAKE"
        mgr.set_session_persistence(lb, fake_type)
        uri = "/loadbalancers/%s/sessionpersistence" % lb.id
        req_body = {"sessionPersistence": {"persistenceType": fake_type}}
        mgr.api.method_put.assert_called_once_with(uri, body=req_body)

    def test_mgr_delete_session_persistence(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_delete = Mock(return_value=({}, {}))
        mgr.delete_session_persistence(lb)
        uri = "/loadbalancers/%s/sessionpersistence" % lb.id
        mgr.api.method_delete.assert_called_once_with(uri)

    def test_mgr_get_connection_logging(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(return_value=({}, {}))
        mgr.get_connection_logging(lb)
        uri = "/loadbalancers/%s/connectionlogging" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_set_connection_logging(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        mgr.set_connection_logging(lb, True)
        uri = "/loadbalancers/%s/connectionlogging" % lb.id
        req_body = {"connectionLogging": {"enabled": "true"}}
        mgr.api.method_put.assert_called_once_with(uri, body=req_body)

    def test_mgr_get_content_caching(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_get = Mock(return_value=({}, {}))
        mgr.get_content_caching(lb)
        uri = "/loadbalancers/%s/contentcaching" % lb.id
        mgr.api.method_get.assert_called_once_with(uri)

    def test_mgr_set_content_caching(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.api.method_put = Mock(return_value=({}, {}))
        mgr.set_content_caching(lb, True)
        uri = "/loadbalancers/%s/contentcaching" % lb.id
        req_body = {"contentCaching": {"enabled": "true"}}
        mgr.api.method_put.assert_called_once_with(uri, body=req_body)

    def test_get_lb(self):
        lb = self.loadbalancer
        mgr = lb.manager
        mgr.get = Mock(return_value=({}, {}))
        mgr._get_lb(lb)
        self.assertEqual(mgr.get.call_count, 0)
        mgr._get_lb(lb.id)
        mgr.get.assert_called_once_with(lb.id)

    def test_bad_node_condition(self):
        self.assertRaises(exc.InvalidNodeCondition, fakes.FakeNode, condition="FAKE")

    def test_bad_node_parameters(self):
        # Can't use FakeNode, since it supplies all valid params.
        self.assertRaises(exc.InvalidNodeParameters, Node)

    def test_node_repr(self):
        nd = fakes.FakeNode(port=12345)
        nd_repr = "%s" % nd
        self.assertTrue("port=12345" in nd_repr)

    def test_node_to_dict(self):
        nd = fakes.FakeNode()
        expected = {"address": nd.address,
                "port": nd.port,
                "condition": nd.condition}
        self.assertEqual(nd.to_dict(), expected)

    def test_node_delete(self):
        lb = self.loadbalancer
        nd = fakes.FakeNode(parent=lb)
        lb.delete_node = Mock()
        nd.delete()
        lb.delete_node.assert_called_once_with(nd)

    def test_node_diff(self):
        nd = fakes.FakeNode()
        new_port = nd.port + 1
        nd.port = new_port
        expected = {"port": new_port}
        self.assertEqual(nd._diff(), expected)

    def test_node_update(self):
        lb = self.loadbalancer
        nd = fakes.FakeNode(parent=lb)
        lb.update_node = Mock()
        nd.update()
        self.assertEqual(lb.update_node.call_count, 0)
        nd.port += 1
        nd.update()
        lb.update_node.assert_called_once_with(nd, {"port": nd.port})

    def test_vip_bad_type(self):
        self.assertRaises(exc.InvalidVirtualIPType, VirtualIP, type="FAKE")

    def test_vip_bad_ip_version(self):
        self.assertRaises(exc.InvalidVirtualIPVersion, VirtualIP, ipVersion="FAKE")

    def test_vip_repr(self):
        vip = fakes.FakeVirtualIP(address="1.2.3.4")
        vip_repr = "%s" % vip
        self.assertTrue("address=1.2.3.4" in vip_repr)

    def test_vip_to_dict(self):
        vip = fakes.FakeVirtualIP()
        expected = {"type": vip.type,
                "ipVersion": vip.ip_version}
        self.assertEqual(vip.to_dict(), expected)

    def test_vip_delete(self):
        lb = self.loadbalancer
        vip = fakes.FakeVirtualIP(parent=lb)
        lb.delete_virtualip = Mock()
        vip.delete()
        lb.delete_virtualip.assert_called_once_with(vip)

    def test_client_create_body(self):
        clt = self.client
        nd = fakes.FakeNode()
        vip = fakes.FakeVirtualIP()
        fake_name = "FAKE"
        fake_port = 999
        fake_protocol = "FAKE"
        fake_nodes = [nd]
        fake_virtual_ips = [vip]
        fake_algorithm = "FAKE"
        expected = {"loadBalancer": {
                "name": fake_name,
                "port": fake_port,
                "protocol": fake_protocol,
                "nodes": [nd.to_dict()],
                "virtualIps": [vip.to_dict()],
                "algorithm": fake_algorithm,
                }}
        ret = clt._create_body(fake_name, port=fake_port, protocol=fake_protocol,
                nodes=fake_nodes, virtual_ips=fake_virtual_ips, algorithm=fake_algorithm)
        self.assertEqual(ret, expected)

    def test_client_get_usage(self):
        clt = self.client
        lb = self.loadbalancer
        clt._manager.get_usage = Mock()
        clt.get_usage(lb)
        clt._manager.get_usage.assert_called_once_with(loadbalancer=lb,
                start=None, end=None)

    def test_client_allowed_domains(self):
        clt = self.client
        fake_name = utils.random_name()
        fake_body = {"allowedDomains": [{"allowedDomain": {"name": fake_name}}]}
        clt.method_get = Mock(return_value=({}, fake_body))
        ret = clt.allowed_domains
        self.assertEqual(ret, [fake_name])
        self.assertEqual(clt.method_get.call_count, 1)
        # Retry; should not re-call the GET
        ret = clt.allowed_domains
        self.assertEqual(ret, [fake_name])
        self.assertEqual(clt.method_get.call_count, 1)

    def test_client_algorithms(self):
        clt = self.client
        fake_name = utils.random_name()
        fake_body = {"algorithms": [{"name": fake_name}]}
        clt.method_get = Mock(return_value=({}, fake_body))
        ret = clt.algorithms
        self.assertEqual(ret, [fake_name])
        self.assertEqual(clt.method_get.call_count, 1)
        # Retry; should not re-call the GET
        ret = clt.algorithms
        self.assertEqual(ret, [fake_name])
        self.assertEqual(clt.method_get.call_count, 1)

    def test_client_protocols(self):
        clt = self.client
        fake_name = utils.random_name()
        fake_body = {"protocols": [{"name": fake_name}]}
        clt.method_get = Mock(return_value=({}, fake_body))
        ret = clt.protocols
        self.assertEqual(ret, [fake_name])
        self.assertEqual(clt.method_get.call_count, 1)
        # Retry; should not re-call the GET
        ret = clt.protocols
        self.assertEqual(ret, [fake_name])
        self.assertEqual(clt.method_get.call_count, 1)





if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2012 Rackspace

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from functools import wraps
import time

import pyrax
from pyrax.client import BaseClient
import pyrax.exceptions as exc
from pyrax.manager import BaseManager
from pyrax.resource import BaseResource
import pyrax.utils as utils


MIN_SIZE = 100
MAX_SIZE = 1024
RETRY_INTERVAL = 5


def _resolve_id(val):
    """Takes an object or an ID and returns the ID."""
    return val if isinstance(val, basestring) else val.id


def _resolve_name(val):
    """Takes an object or a name and returns the name."""
    return val if isinstance(val, basestring) else val.name


def assure_volume(fnc):
    """Converts a volumeID passed as the volume to a CloudBlockStorageVolume object."""
    @wraps(fnc)
    def _wrapped(self, volume, *args, **kwargs):
        if not isinstance(volume, CloudBlockStorageVolume):
            # Must be the ID
            volume = self._manager.get(volume)
        return fnc(self, volume, *args, **kwargs)
    return _wrapped


def assure_snapshot(fnc):
    """Converts a snapshot ID passed as the snapshot to a CloudBlockStorageSnapshot object."""
    @wraps(fnc)
    def _wrapped(self, snapshot, *args, **kwargs):
        if not isinstance(snapshot, CloudBlockStorageSnapshot):
            # Must be the ID
            snapshot = self._snaps_manager.get(snapshot)
        return fnc(self, snapshot, *args, **kwargs)
    return _wrapped


class CloudBlockStorageSnapshot(BaseResource):
    """
    This class represents a Snapshot (copy) of a Block Storage Volume.
    """
    def delete(self):
        """
        Adds a check to make sure that the snapshot is able to be deleted.
        """
        if not self.status in ("available", "error"):
            raise exc.SnapshotNotAvailable("Snapshot must be in 'available' or 'error' status "
                    "before deleting. Current status: %s" % self.status)
        # When there are more thann one snapshot for a given volume, attempting to
        # delete them all will throw a 409 exception. This will help by retrying
        # such an error once after a RETRY_INTERVAL second delay.
        try:
            super(CloudBlockStorageSnapshot, self).delete()
        except exc.ClientException as e:
            if "Request conflicts with in-progress 'DELETE" in str(e):
                time.sleep(RETRY_INTERVAL)
                # Try again; if it fails, oh, well...
                super(CloudBlockStorageSnapshot, self).delete()


    def _get_name(self):
        return self.display_name

    def _set_name(self, val):
        self.display_name = val

    name = property(_get_name, _set_name, None,
            "Convenience for referencing the display_name.")

    def _get_description(self):
        return self.display_description

    def _set_description(self, val):
        self.display_description = val

    description = property(_get_description, _set_description, None,
            "Convenience for referencing the display_description.")


class CloudBlockStorageVolumeType(BaseResource):
    """
    This class represents a Block Storage Volume Type.
    """
    pass


class CloudBlockStorageVolume(BaseResource):
    """
    This class represents a Block Storage volume.
    """
    def __init__(self, *args, **kwargs):
        super(CloudBlockStorageVolume, self).__init__(*args, **kwargs)
        try:
            self._nova_volumes = pyrax.cloudservers.volumes
        except AttributeError:
            # This will happen in unit testing, where the full pyrax
            # namespace is not exposed. In that situation, there is
            # no need for the reference anyway
            pass
        self._snapshot_manager = BaseManager(self.manager.api,
                resource_class=CloudBlockStorageSnapshot,
                response_key="snapshot", uri_base="snapshots")


    def attach_to_instance(self, instance, mountpoint):
        """
        Attaches this volume to the cloud server instance at the
        specified mountpoint. This requires a call to the cloud servers
        API; it cannot be done directly.
        """
        instance_id = _resolve_id(instance)
        resp = self._nova_volumes.create_server_volume(instance_id,
                self.id, mountpoint)
        # The response should be a volume reference to this volume.
        return resp.id == self.id


    def detach(self):
        """
        Detaches this volume from any device it may be attached to. If it
        is not attached, nothing happens.
        """
        attachments = self.attachments
        if not attachments:
            # Not attached; no error needed, just return
            return
        # A volume can only be attached to one device at a time, but for some
        # reason this is a list instead of a singular value
        att = attachments[0]
        instance_id = att["server_id"]
        attachment_id = att["id"]
        self._nova_volumes.delete_server_volume(instance_id, attachment_id)


    def delete(self, force=False):
        """Volumes cannot be deleted if either a) they are attached to a device,
        or b) they have any snapshots. This method overrides the base delete()
        method to both better handle these failures, and also to offer a 'force'
        option. When 'force' is True, the volume is detached, and any dependent
        snapshots are deleted before calling the volume's delete.
        """
        if force:
            self.detach()
            self.delete_all_snapshots()
        try:
            super(CloudBlockStorageVolume, self).delete()
        except exc.VolumeNotAvailable:
            # Notify the user? Record it somewhere? 
            # For now, just re-raise
            raise


    def create_snapshot(self, name=None, description=None, force=False):
        """
        Creates a snapshot of this volume, with an optional name and description.

        Normally snapshots will not happen if the volume is attached. To override
        this default behavior, pass force=True.
        """
        name = name or ""
        description = description or ""
        # Note that passing in non-None values is required for the _create_body
        # method to distinguish between this and the request to create and instance.
        try:
            snap = self._snapshot_manager.create(volume=self, name=name, description=description,
                    force=force)
        except exc.BadRequest as e:
            msg = str(e)
            if "Invalid volume: must be available" in msg:
                # The volume for the snapshot was attached.
                raise exc.VolumeNotAvailable("Cannot create a snapshot from an attached volume. "
                        "Detach the volume before trying again, or pass 'force=True' to the "
                        "create_snapshot() call.")
            else:
                # Some other error
                raise
        except exc.ClientException as e:
            if e.code == 409:
                if "Request conflicts with in-progress" in str(e):
                    raise exc.VolumeNotAvailable("The volume is current creating a snapshot. You "
                            "must wait until that completes before attempting to create an "
                            "additional snapshot.")
                else:
                    raise
            else:
                raise
        return snap


    def list_snapshots(self):
        """
        Returns a list of all snapshots of this volume.
        """
        return [snap for snap in self._snapshot_manager.list()
                if snap.volume_id == self.id]


    def delete_all_snapshots(self):
        """
        Locates all snapshots of this volume and deletes them.
        """
        for snap in self.list_snapshots():
            snap.delete()


    def _get_name(self):
        return self.display_name

    def _set_name(self, val):
        self.display_name = val

    name = property(_get_name, _set_name, None,
            "Convenience for referencing the display_name.")

    def _get_description(self):
        return self.display_description

    def _set_description(self, val):
        self.display_description = val

    description = property(_get_description, _set_description, None,
            "Convenience for referencing the display_description.")


class CloudBlockStorageClient(BaseClient):
    """
    This is the primary class for interacting with Cloud Block Storage.
    """
    def _configure_manager(self):
        """
        Create the manager to handle the instances, and also another
        to handle flavors.
        """
        self._manager = BaseManager(self, resource_class=CloudBlockStorageVolume,
               response_key="volume", uri_base="volumes")
        self._types_manager = BaseManager(self, resource_class=CloudBlockStorageVolumeType,
               response_key="volume_type", uri_base="types")
        self._snaps_manager = BaseManager(self, resource_class=CloudBlockStorageSnapshot,
               response_key="snapshot", uri_base="snapshots")


    def create(self, name="", size=None, volume_type=None, description=None,
             metadata=None, snapshot_id=None, availability_zone=None):
        """
        Makes sure that the size is passed and is within allowed values.
        """
        if not isinstance(size, (int, long)) or not MIN_SIZE <= size <= MAX_SIZE:
            raise exc.InvalidSize("Volume sizes must be integers between %s and %s." %
                    (MIN_SIZE, MAX_SIZE))
        return super(CloudBlockStorageClient, self).create(name, size=size,
                volume_type=volume_type, description=description, metadata=metadata,
                snapshot_id=snapshot_id, availability_zone=availability_zone)


    def list_types(self):
        """Returns a list of all available volume types."""
        return self._types_manager.list()


    def list_snapshots(self):
        """Returns a list of all snapshots."""
        return self._snaps_manager.list()


    def _create_body(self, name, size=None, volume_type=None, description=None,
             metadata=None, snapshot_id=None, availability_zone=None,
             volume=None, force=False):
        """
        Used to create the dict required to create any of the following:
            A new volume
            A new snapshot
        """
        if size is not None:
            # Creating a volume
            if not isinstance(size, (int, long)) or not MIN_SIZE <= size <= MAX_SIZE:
                raise exc.InvalidSize("Volume sizes must be integers between %s and %s." %
                        (MIN_SIZE, MAX_SIZE))
            if volume_type is None:
                volume_type = "SATA"
            if description is None:
                description = ""
            if metadata is None:
                metadata = {}
            body = {"volume": {
                    "size": size,
                    "snapshot_id": snapshot_id,
                    "display_name": name,
                    "display_description": description,
                    "volume_type": volume_type,
                    "metadata": metadata,
                    "availability_zone": availability_zone,
                    }}
        else:
            # Creating a snapshot
            body = {"snapshot": {
                    "display_name": name,
                    "display_description": description,
                    "volume_id": volume.id,
                    "force": str(force).lower(),
                 }}
        return body


    @assure_volume
    def attach_to_instance(self, volume, instance, mountpoint):
        """Attaches the volume to the specified instance at the mountpoint."""
        return volume.attach_to_instance(instance, mountpoint)


    @assure_volume
    def detach(self, volume):
        """Detaches the volume from whatever device it is attached to."""
        return volume.detach()


    @assure_volume
    def delete_volume(self, volume, force=False):
        """Deletes the volume."""
        return volume.delete(force=force)


    @assure_volume
    def create_snapshot(self, volume, name=None, description=None, force=False):
        """
        Creates a snapshot of the volume, with an optional name and description.

        Normally snapshots will not happen if the volume is attached. To override
        this default behavior, pass force=True.
        """
        return volume.create_snapshot(name=name, description=description, force=force)


    @assure_snapshot
    def delete_snapshot(self, snapshot):
        """Deletes the snapshot."""
        return snapshot.delete()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import os
import unittest

from mock import patch
from mock import MagicMock as Mock

import pyrax
import pyrax.exceptions as exc
import pyrax.utils as utils
from tests.unit import fakes



class PyraxInitTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        reload(pyrax)
        # Override any config settings that turn off services.
        for key, val in pyrax.services_to_start.items():
            pyrax.services_to_start[key] = True
        self.orig_connect_to_cloudservers = pyrax.connect_to_cloudservers
        self.orig_connect_to_cloudfiles = pyrax.connect_to_cloudfiles
        self.orig_connect_to_cloud_loadbalancers = pyrax.connect_to_cloud_loadbalancers
        self.orig_connect_to_cloud_databases = pyrax.connect_to_cloud_databases
        self.orig_get_service_endpoint = pyrax._get_service_endpoint
        super(PyraxInitTest, self).__init__(*args, **kwargs)
        self.username = "fakeuser"
        self.api_key = "fakeapikey"

    def setUp(self):
        pyrax.set_identity_class(fakes.FakeIdentity)
        pyrax.identity = pyrax.identity_class()
        pyrax.identity.authenticated = True
        pyrax.connect_to_cloudservers = Mock()
        pyrax.connect_to_cloudfiles = Mock()
        pyrax.connect_to_cloud_loadbalancers = Mock()
        pyrax.connect_to_cloud_databases = Mock()
        pyrax._get_service_endpoint = Mock(return_value="http://example.com/")

    def tearDown(self):
        pyrax.connect_to_cloudservers = self.orig_connect_to_cloudservers
        pyrax.connect_to_cloudfiles = self.orig_connect_to_cloudfiles
        pyrax.connect_to_cloud_loadbalancers = self.orig_connect_to_cloud_loadbalancers
        pyrax.connect_to_cloud_databases = self.orig_connect_to_cloud_databases
        pyrax._get_service_endpoint = self.orig_get_service_endpoint

    def test_require_auth(self):
        @pyrax._require_auth
        def testfunc(): pass
        pyrax.identity.authenticated = True
        testfunc()
        pyrax.identity.authenticated = False
        self.assertRaises(exc.NotAuthenticated, testfunc)

    def test_set_credentials(self):
        pyrax.set_credentials(self.username, self.api_key)
        self.assertEqual(pyrax.identity.username, self.username)
        self.assertEqual(pyrax.identity.api_key, self.api_key)
        self.assert_(pyrax.identity.authenticated)

    def test_set_bad_credentials(self):
        self.assertRaises(exc.AuthenticationFailed, pyrax.set_credentials, "bad", "creds")
        self.assertFalse(pyrax.identity.authenticated)

    def test_set_credential_file(self):
        with utils.SelfDeletingTempfile() as tmpname:
            with file(tmpname, "wb") as tmp:
                tmp.write("[rackspace_cloud]\n")
                tmp.write("username = %s\n" % self.username)
                tmp.write("api_key = %s\n" % self.api_key)
            pyrax.set_credential_file(tmpname)
            self.assertEqual(pyrax.identity.username, self.username)
            self.assertEqual(pyrax.identity.api_key, self.api_key)
            self.assert_(pyrax.identity.authenticated)

    def test_set_bad_credential_file(self):
        with utils.SelfDeletingTempfile() as tmpname:
            with file(tmpname, "wb") as tmp:
                tmp.write("[rackspace_cloud]\n")
                tmp.write("username = bad\n")
                tmp.write("api_key = creds\n")
            self.assertRaises(exc.AuthenticationFailed, pyrax.set_credential_file, tmpname)
            self.assertFalse(pyrax.identity.authenticated)

    def test_clear_credentials(self):
        pyrax.set_credentials(self.username, self.api_key)
        # These next lines are required to test that clear_credentials
        # actually resets them to None.
        pyrax.cloudservers = object()
        pyrax.cloudfiles = object()
        pyrax.cloud_loadbalancers = object()
        pyrax.cloud_databases = object()
        default_region = object()
        self.assert_(pyrax.identity.authenticated)
        self.assertIsNotNone(pyrax.cloudfiles)
        pyrax.clear_credentials()
        self.assertFalse(pyrax.identity.authenticated)
        self.assertIsNone(pyrax.identity.username)
        self.assertIsNone(pyrax.identity.api_key)
        self.assertIsNone(pyrax.cloudservers)
        self.assertIsNone(pyrax.cloudfiles)
        self.assertIsNone(pyrax.cloud_loadbalancers)
        self.assertIsNone(pyrax.cloud_databases)

    def test_set_default_region(self):
        orig_region = pyrax.default_region
        new_region = "test"
        pyrax.set_default_region(new_region)
        self.assertEqual(pyrax.default_region, new_region)

    def test_fix_uri_other(self):
        region = "abc"
        svc = "other"
        orig = "http://example.com/v1.0/fake"
        expected = "http://abc.example.com/v1.0/fake"
        fixed = pyrax._fix_uri(orig, region, svc)
        self.assertEqual(fixed, expected)

    def test_fix_uri_compute(self):
        region = "abc"
        svc = "compute"
        orig = "http://example.com/v1.0/fake"
        expected = "http://abc.example.com/v2/fake"
        fixed = pyrax._fix_uri(orig, region, svc)
        self.assertEqual(fixed, expected)

    def test_make_agent_name(self):
        test_agent = "TEST"
        ret = pyrax._make_agent_name(test_agent)
        self.assert_(ret.startswith(test_agent))
        self.assert_(ret.endswith(pyrax.USER_AGENT))

    def test_connect_to_services(self):
        pyrax.connect_to_services()
        pyrax.connect_to_cloudservers.assert_called_once_with()
        pyrax.connect_to_cloudfiles.assert_called_once_with()
        pyrax.connect_to_cloud_loadbalancers.assert_called_once_with()
        pyrax.connect_to_cloud_databases.assert_called_once_with()

    @patch('pyrax._cs_client.Client', new=fakes.FakeService)
    def test_connect_to_cloudservers(self):
        pyrax.cloudservers = None
        pyrax.connect_to_cloudservers = self.orig_connect_to_cloudservers
        pyrax.cloudservers = pyrax.connect_to_cloudservers()
        self.assertIsNotNone(pyrax.cloudservers)

    @patch('pyrax._cf.CFClient', new=fakes.FakeService)
    def test_connect_to_cloudfiles(self):
        pyrax.cloudfiles = None
        pyrax.connect_to_cloudfiles = self.orig_connect_to_cloudfiles
        pyrax.cloudfiles = pyrax.connect_to_cloudfiles()
        self.assertIsNotNone(pyrax.cloudfiles)

    @patch('pyrax.CloudLoadBalancerClient', new=fakes.FakeService)
    def test_connect_to_cloud_loadbalancers(self):
        pyrax.cloud_loadbalancers = None
        pyrax.connect_to_cloud_loadbalancers = self.orig_connect_to_cloud_loadbalancers
        pyrax.cloud_loadbalancers = pyrax.connect_to_cloud_loadbalancers()
        self.assertIsNotNone(pyrax.cloud_loadbalancers)

    @patch('pyrax.CloudDatabaseClient', new=fakes.FakeService)
    def test_connect_to_cloud_databases(self):
        pyrax.cloud_databases = None
        pyrax.connect_to_cloud_databases = self.orig_connect_to_cloud_databases
        pyrax.cloud_databases = pyrax.connect_to_cloud_databases()
        self.assertIsNotNone(pyrax.cloud_databases)

    def test_import_fail(self):
        import __builtin__
        sav_import = __builtin__.__import__
        def fake_import(nm, *args):
            if nm == "rax_identity":
                raise ImportError
            else:
                return sav_import(nm, *args)

        __builtin__.__import__ = fake_import
        self.assertRaises(ImportError, reload, pyrax)
        __builtin__.__import__ = sav_import
        reload(pyrax)



if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2012 Rackspace

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from functools import wraps
from pyrax.client import BaseClient
import pyrax.exceptions as exc
from pyrax.manager import BaseManager
from pyrax.resource import BaseResource
import pyrax.utils as utils


def assure_instance(fnc):
    @wraps(fnc)
    def _wrapped(self, instance, *args, **kwargs):
        if not isinstance(instance, CloudDatabaseInstance):
            # Must be the ID
            instance = self._manager.get(instance)
        return fnc(self, instance, *args, **kwargs)
    return _wrapped


class CloudDatabaseInstance(BaseResource):
    """
    This class represents a MySQL instance in the cloud. 
    """
    def __init__(self, *args, **kwargs):
        super(CloudDatabaseInstance, self).__init__(*args, **kwargs)
        self._database_manager = BaseManager(self.manager.api,
                resource_class=CloudDatabaseDatabase, response_key="database",
                uri_base="instances/%s/databases" % self.id)
        self._user_manager = BaseManager(self.manager.api,
                resource_class=CloudDatabaseUser, response_key="user",
                uri_base="instances/%s/users" % self.id)


    def list_databases(self):
        """Returns a list of the names of all databases for this instance."""
        return self._database_manager.list()


    def list_users(self):
        """Returns a list of the names of all users for this instance."""
        return self._user_manager.list()


    def get_database(self, name):
        """
        Finds the database in this instance with the specified name, and
        returns a CloudDatabaseDatabase object. If no match is found, a
        NoSuchDatabase exception is raised.
        """
        try:
            return [db for db in self.list_databases()
                    if db.name == name][0]
        except IndexError:
            raise exc.NoSuchDatabase("No database by the name '%s' exists." % name)


    def get_user(self, name):
        """
        Finds the user in this instance with the specified name, and
        returns a CloudDatabaseUser object. If no match is found, a
        NoSuchDatabaseUser exception is raised.
        """
        try:
            return [user for user in self.list_users()
                    if user.name == name][0]
        except IndexError:
            raise exc.NoSuchDatabaseUser("No user by the name '%s' exists." % name)


    def create_database(self, name, character_set=None, collate=None):
        """
        Creates a database with the specified name. If a database with
        that name already exists, a BadRequest (400) exception will
        be raised.
        """
        if character_set is None:
            character_set = "utf8"
        if collate is None:
            collate = "utf8_general_ci"
        # Note that passing in non-None values is required for the _create_body
        # method to distinguish between this and the request to create and instance.
        self._database_manager.create(name=name, character_set=character_set,
                collate=collate, return_none=True)
        # Since the API doesn't return the info for creating the database object, we
        # have to do it manually.
        return self._database_manager.find(name=name)


    def create_user(self, name, password, database_names):
        """
        Creates a user with the specified name and password, and gives that
        user access to the specified database(s).

        If a user with
        that name already exists, a BadRequest (400) exception will
        be raised.
        """
        if not isinstance(database_names, list):
            database_names = [database_names]
        # The API only accepts names, not DB objects
        database_names = [db if isinstance(db, basestring) else db.name
                for db in database_names]
        # Note that passing in non-None values is required for the create_body
        # method to distinguish between this and the request to create and instance.
        self._user_manager.create(name=name, password=password,
                database_names=database_names, return_none=True)
        # Since the API doesn't return the info for creating the user object, we
        # have to do it manually.
        return self._user_manager.find(name=name)


    def _get_name(self, name_or_obj):
        """
        For convenience, many methods accept either an object or the name
        of the object as a parameter, but need the name to send to the
        API. This method handles that conversion.
        """
        if isinstance(name_or_obj, basestring):
            return name_or_obj
        try:
            return name_or_obj.name
        except AttributeError:
            msg = "The object '%s' does not have a 'name' attribute." % name_or_obj
            raise exc.MissingName(msg)


    def delete_database(self, name_or_obj):
        """
        Deletes the specified database. If no database by that name
        exists, no exception will be raised; instead, nothing at all
        is done.
        """
        name = self._get_name(name_or_obj)
        self._database_manager.delete(name)


    def delete_user(self, name_or_obj):
        """
        Deletes the specified user. If no user by that name
        exists, no exception will be raised; instead, nothing at all
        is done.
        """
        name = self._get_name(name_or_obj)
        self._user_manager.delete(name)


    def enable_root_user(self):
        """
        Enables login from any host for the root user and provides
        the user with a generated root password.
        """
        uri = "/instances/%s/root" % self.id
        resp, body = self.manager.api.method_post(uri)
        return body["user"]["password"]


    def root_user_status(self):
        """
        Returns True or False, depending on whether the root user
        for this instance has been enabled.
        """
        uri = "/instances/%s/root" % self.id
        resp, body = self.manager.api.method_get(uri)
        return body["rootEnabled"]


    def restart(self):
        """Restarts this instance."""
        self.manager.action(self, "restart")


    def resize(self, flavor):
        """Set the size of this instance to a different flavor."""
        # We need the flavorRef, not the flavor or size.
        flavorRef = self.manager.api._get_flavor_ref(flavor)
        body = {"flavorRef": flavorRef}
        self.manager.action(self, "resize", body=body)


    def resize_volume(self, size):
        """Changes the size of the volume for this instance."""
        curr_size = self.volume.get("size")
        if size <= curr_size:
            raise exc.InvalidVolumeResize("The new volume size must be larger than the current volume size of '%s'." % curr_size)
        body = {"volume": {"size": size}}
        self.manager.action(self, "resize", body=body)


    def _get_flavor(self):
        try:
            ret = self._flavor
        except AttributeError:
            ret = self._flavor = CloudDatabaseFlavor(self.manager.api._flavor_manager, {})
        return ret

    def _set_flavor(self, flavor):
        if isinstance(flavor, dict):
            self._flavor = CloudDatabaseFlavor(self.manager.api._flavor_manager, flavor)
        else:
            # Must be an instance
            self._flavor = flavor

    flavor = property(_get_flavor, _set_flavor)


class CloudDatabaseDatabase(BaseResource):
    """
    This class represents a database on a CloudDatabaseInstance. It is not
    a true cloud entity, but a convenience object for dealing with databases
    on instances.
    """
    def delete(self):
        """This class doesn't have an 'id', so pass the name."""
        self.manager.delete(self.name)


class CloudDatabaseUser(BaseResource):
    """
    This class represents a user on a CloudDatabaseInstance. It is not
    a true cloud entity, but a convenience object for dealing with users
    for instances.
    """
    def delete(self):
        """This class doesn't have an 'id', so pass the name."""
        self.manager.delete(self.name)


class CloudDatabaseFlavor(BaseResource):
    """
    This class represents the available instance configurations, or 'flavors',
    which you use to define the memory and CPU size of your instance. These
    objects are read-only.
    """
    pass


class CloudDatabaseClient(BaseClient):
    """
    This is the primary class for interacting with Cloud Databases.
    """
    def _configure_manager(self):
        """
        Creates a manager to handle the instances, and another
        to handle flavors.
        """
        self._manager = BaseManager(self, resource_class=CloudDatabaseInstance,
               response_key="instance", uri_base="instances")
        self._flavor_manager = BaseManager(self,
                resource_class=CloudDatabaseFlavor, response_key="flavor",
                uri_base="flavors")


    @assure_instance
    def list_databases(self, instance):
        """Returns all databases for the specified instance."""
        return instance.list_databases()


    @assure_instance
    def create_database(self, instance, name, character_set=None,
            collate=None):
        """Creates a database with the specified name on the given instance."""
        return instance.create_database(name, character_set=character_set,
                collate=collate)


    @assure_instance
    def get_database(self, instance, name):
        """
        Finds the database in the given instance with the specified name, and
        returns a CloudDatabaseDatabase object. If no match is found, a
        NoSuchDatabase exception is raised.
        """
        return instance.get_database(name)


    @assure_instance
    def delete_database(self, instance, name):
        """Deletes the database by name on the given instance."""
        return instance.delete_database(name)


    @assure_instance
    def list_users(self, instance):
        """Returns all users for the specified instance."""
        return instance.list_users()


    @assure_instance
    def create_user(self, instance, name, password, database_names):
        """
        Creates a user with the specified name and password, and gives that
        user access to the specified database(s).
        """
        return instance.create_user(name=name, password=password,
                database_names=database_names)


    @assure_instance
    def get_user(self, instance, name):
        """
        Finds the user in the given instance with the specified name, and
        returns a CloudDatabaseUser object. If no match is found, a
        NoSuchUser exception is raised.
        """
        return instance.get_user(name)


    @assure_instance
    def delete_user(self, instance, name):
        """Deletes the user by name on the given instance."""
        return instance.delete_user(name)


    @assure_instance
    def enable_root_user(self, instance):
        """
        This enables login from any host for the root user and provides
        the user with a generated root password.
        """
        return instance.enable_root_user()


    @assure_instance
    def root_user_status(self, instance):
        """Returns True if the given instance is root-enabled."""
        return instance.root_user_status()


    @assure_instance
    def restart(self, instance):
        """Restarts the instance."""
        return instance.restart()


    @assure_instance
    def resize(self, instance, flavor):
        """Sets the size of the instance to a different flavor."""
        return instance.resize(flavor)


    def list_flavors(self):
        """Returns a list of all available Flavors."""
        return self._flavor_manager.list()


    def get_flavor(self, flavor_id):
        """Returns a specific Flavor object by ID."""
        return self._flavor_manager.get(flavor_id)


    def _get_flavor_ref(self, flavor):
        """
        Flavors are odd in that the API expects an href link, not
        an ID, as with nearly every other resource. This method
        takes either a CloudDatabaseFlavor object, a flavor ID,
        a RAM size, or a flavor name, and uses that to determine
        the appropriate href.
        """
        flavor_obj = None
        if isinstance(flavor, CloudDatabaseFlavor):
            flavor_obj = flavor
        elif isinstance(flavor, int):
            # They passed an ID or a size
            try:
                flavor_obj = self.get_flavor(flavor)
            except exc.NotFound:
                # Must be either a size or bad ID, which will
                # be handled below
                pass
        if flavor_obj is None:
            # Try flavor name
            flavors = self.list_flavors()
            try:
                flavor_obj = [flav for flav in flavors
                        if flav.name == flavor][0]
            except IndexError:
                # No such name; try matching RAM
                try:
                    flavor_obj = [flav for flav in flavors
                            if flav.ram == flavor][0]
                except IndexError:
                   raise exc.FlavorNotFound("Could not determine flavor from '%s'." % flavor)
        # OK, we have a Flavor object. Get the href
        href = [link["href"] for link in flavor_obj.links
                if link["rel"] == "self"][0]
        return href


    def _create_body(self, name, flavor=None, volume=None, databases=None,
            users=None, character_set=None, collate=None, password=None,
            database_names=None):
        """
        Used to create the dict required to create any of the following:
            A database instance
            A database for an instance
            A user for an instance
        """
        if character_set is not None:
            # Creating a database
            body = {"databases": [
                    {"name": name,
                    "character_set": character_set,
                    "collate": collate,
                    }]}
        elif password is not None:
            # Creating a user
            db_dicts = [{"name": db} for db in database_names]
            body = {"users": [
                    {"name": name,
                    "password": password,
                    "databases": db_dicts,
                    }]}
        else:
            if flavor is None:
                flavor = 1
            flavor_ref = self._get_flavor_ref(flavor)
            if volume is None:
                volume = 1
            if databases is None:
                databases = []
            if users is None:
                users = []
            body = {"instance": {
                    "name": name,
                    "flavorRef": flavor_ref,
                    "volume": {"size": volume},
                    "databases": databases,
                    "users": users,
                    }}
        return body



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2012 Rackspace

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import pyrax

creds_file = os.path.expanduser("~/.rackspace_cloud_credentials")
pyrax.set_credential_file(creds_file)
cf = pyrax.cloudfiles

cont_name = pyrax.utils.random_name(8)
cont = cf.create_container(cont_name)
obj_name = pyrax.utils.random_name(8)

text = "This is some text containing unicode like é, ü and ˚¬∆ç"
obj = cf.store_object(cont, obj_name, text)

# Verify that the object is there
print "Stored Object Name:", obj.name
print "Size:", obj.total_bytes

# Make sure that the content stored is identical
stored_text = obj.get()
print "Original text:", text
print "  Stored text:", stored_text
if stored_text == text:
    print "Stored text is identical"
else:
    print "Difference detected!"
    print "Original:", text
    print "Stored:", stored_text

# Clean up
cont.delete(True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2012 Rackspace

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import time

import pyrax
import pyrax.exceptions as exc
import pyrax.utils as utils

creds_file = os.path.expanduser("~/.rackspace_cloud_credentials")
pyrax.set_credential_file(creds_file)
cf = pyrax.cloudfiles

cont_name = pyrax.utils.random_name(8)
cont = cf.create_container(cont_name)

# pyrax has a utility for creating temporary local directories that clean themselves up.
with utils.SelfDeletingTempDirectory() as tmpfolder:
    # Create a bunch of files
    for idx in xrange(13):
        fname = "file_%s" % idx
        pth = os.path.join(tmpfolder, fname)
        with file(pth, "w") as tmp:
            tmp.write("This is some text")
    # Create a subfolder. It will be deleted automatically as part of
    # the cleanup of SelfDeletingTempDirectory.
    subfolder_path = os.path.join(tmpfolder, "subfolder")
    os.mkdir(subfolder_path)
    # Create some files in the subfolder, too.
    for idx in xrange(7):
        fname = "subfile_%s" % idx
        pth = os.path.join(subfolder_path, fname)
        with file(pth, "w") as tmp:
            tmp.write("This is some text. " * 100)

    # OK, we've created our local file system. Now upload it to a container
    # named 'upfolder'. We'll have it skip all files ending in the digits
    # '2', '6' or '0'.
    ignore = ["*2", "*6", "*0"]
    print "Beginning Folder Uplaod"
    upload_key, total_bytes = cf.upload_folder(tmpfolder, cont, ignore=ignore)
    # Since upload_folder happens in the background, we need to stay in this
    # block until the upload is complete, or the SelfDeletingTempDirectory
    # will be deleted, and the upload won't find the files it needs.
    print "Total bytes to upload:", total_bytes
    uploaded = 0
    while uploaded < total_bytes:
        uploaded = cf.get_uploaded(upload_key)
        print "Progress: %4.2f%%" % ((uploaded * 100.0) / total_bytes)
        time.sleep(1)

# OK, the upload is complete. Let's verify what's in 'upfolder'.
folder_name = os.path.basename(tmpfolder)
print 
print "Temp folder name:", folder_name
nms = cf.get_container_object_names(cont, prefix=folder_name)
print "Number of files in container:", len(nms)
print "\n".join(nms)

# Clean up
cont.delete(True)



# Copyright 2010 Jacob Kaplan-Moss

# Copyright 2011 OpenStack LLC.
# Copyright 2012 Rackspace

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Base utilities to build API operation managers and objects on top of.
"""

import contextlib
import hashlib
import os

import pyrax.exceptions as exc
import pyrax.utils as utils


# Python 2.4 compat
try:
    all
except NameError:
    def all(iterable):
        return True not in (not x for x in iterable)


def getid(obj):
    """
    Abstracts the common pattern of allowing both an object or an object's ID
    as a parameter when dealing with relationships.
    """
    try:
        return obj.id
    except AttributeError:
        return obj



class BaseManager(object):
    """
    Managers interact with a particular type of API (servers, databases, dns,
    etc.) and provide CRUD operations for them.
    """
    resource_class = None
    response_key = None
    plural_response_key = None
    uri_base = None
    _hooks_map = {}


    def __init__(self, api, resource_class=None, response_key=None,
            plural_response_key=None, uri_base=None):
        self.api = api
        self.resource_class = resource_class
        self.response_key = response_key
        if plural_response_key:
            self.plural_response_key = plural_response_key
        else:
            # Default to adding 's'
            self.plural_response_key = "%ss" % response_key
        self.uri_base = uri_base


    def list(self):
        """Gets a list of all items."""
        return self._list("/%s" % self.uri_base)


    def get(self, item):
        """Gets a specific item."""
        uri = "/%s/%s" % (self.uri_base, getid(item))
        return self._get(uri)


    def create(self, name, return_none=False, return_raw=False, *args, **kwargs):
        """
        Subclasses need to implement the _create_body() method
        to return a dict that will be used for the API request
        body.
        """
        body = self.api._create_body(name, *args, **kwargs)
        return self._create("/%s" % self.uri_base, body, return_none=return_none,
                return_raw=return_raw)


    def delete(self, item):
        """Deletes the specified item."""
        uri = "/%s/%s" % (self.uri_base, getid(item))
        return self._delete(uri)


    def _list(self, url, obj_class=None, body=None):
        """
        Handles the communication with the API when getting
        a full listing of the resources managed by this class.
        """
        if body:
            _resp, body = self.api.method_post(url, body=body)
        else:
            _resp, body = self.api.method_get(url)

        if obj_class is None:
            obj_class = self.resource_class

        data = body[self.plural_response_key]
        # NOTE(ja): keystone returns values as list as {"values": [ ... ]}
        #           unlike other services which just return the list...
        if isinstance(data, dict):
            try:
                data = data["values"]
            except KeyError:
                pass
        return [obj_class(self, res, loaded=False)
                for res in data if res]


    def _get(self, url):
        """
        Handles the communication with the API when getting
        a specific resource managed by this class.
        """
        _resp, body = self.api.method_get(url)
        return self.resource_class(self, body[self.response_key], loaded=True)


    def _create(self, url, body, return_none=False, return_raw=False, **kwargs):
        """
        Handles the communication with the API when creating a new
        resource managed by this class.
        """
        self.run_hooks("modify_body_for_create", body, **kwargs)
        _resp, body = self.api.method_post(url, body=body)
        if return_none:
            # No response body
            return
        if return_raw:
            return body[self.response_key]
        return self.resource_class(self, body[self.response_key])


    def _delete(self, url):
        """
        Handles the communication with the API when deleting
        a specific resource managed by this class.
        """
        _resp, _body = self.api.method_delete(url)


    def _update(self, url, body, **kwargs):
        """
        Handles the communication with the API when updating
        a specific resource managed by this class.
        """
        self.run_hooks("modify_body_for_update", body, **kwargs)
        _resp, body = self.api.method_put(url, body=body)
        return body


    def action(self, item, action_type, body={}):
        """
        Several API calls are lumped under the 'action' API. This
        is the generic handler for such calls.
        """
        uri = "/%s/%s/action" % (self.uri_base, getid(item))
        action_body = {action_type: body}
        return self.api.method_post(uri, body=action_body)


    def find(self, **kwargs):
        """
        Finds a single item with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        matches = self.findall(**kwargs)
        num_matches = len(matches)
        if not num_matches:
            msg = "No %s matching: %s." % (self.resource_class.__name__, kwargs)
            raise exc.NotFound(404, msg)
        if num_matches > 1:
            msg = "More than one %s matching: %s." % (self.resource_class.__name__, kwargs)
            raise exc.NoUniqueMatch(400, msg)
        else:
            return matches[0]


    def findall(self, **kwargs):
        """
        Finds all items with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        found = []
        searches = kwargs.items()

        for obj in self.list():
            try:
                if all(getattr(obj, attr) == value
                        for (attr, value) in searches):
                    found.append(obj)
            except AttributeError:
                continue
        return found


    @classmethod
    def add_hook(cls, hook_type, hook_func):
        if hook_type not in cls._hooks_map:
            cls._hooks_map[hook_type] = []

        cls._hooks_map[hook_type].append(hook_func)


    @classmethod
    def run_hooks(cls, hook_type, *args, **kwargs):
        hook_funcs = cls._hooks_map.get(hook_type) or []
        for hook_func in hook_funcs:
            hook_func(*args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import hashlib
import os
import StringIO
import sys
import unittest

from mock import patch
from mock import MagicMock as Mock

import pyrax.utils as utils
import pyrax.exceptions as exc
from tests.unit import fakes



class UtilsTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(UtilsTest, self).__init__(*args, **kwargs)

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_self_deleting_temp_file(self):
        with utils.SelfDeletingTempfile() as tmp:
            self.assert_(isinstance(tmp, basestring))
            self.assert_(os.path.exists(tmp))
            self.assert_(os.path.isfile(tmp))
        # File shoud be deleted after exiting the block
        self.assertFalse(os.path.exists(tmp))

    def test_self_deleting_temp_directory(self):
        with utils.SelfDeletingTempDirectory() as tmp:
            self.assert_(isinstance(tmp, basestring))
            self.assert_(os.path.exists(tmp))
            self.assert_(os.path.isdir(tmp))
        # Directory shoud be deleted after exiting the block
        self.assertFalse(os.path.exists(tmp))

    def test_get_checksum_from_string(self):
        test = "some random text"
        md = hashlib.md5()
        md.update(test)
        expected = md.hexdigest()
        received = utils.get_checksum(test)
        self.assertEqual(expected, received)

    def test_get_checksum_from_file(self):
        test = "some random text"
        md = hashlib.md5()
        md.update(test)
        expected = md.hexdigest()
        with utils.SelfDeletingTempfile() as tmp:
            with file(tmp, "w") as testfile:
                testfile.write(test)
            with file(tmp, "r") as testfile:
                received = utils.get_checksum(testfile)
        self.assertEqual(expected, received)

    def test_random_name(self):
        nm = utils.random_name(33)
        self.assertEqual(len(nm), 33)
        nm = utils.random_name(9999)
        self.assertEqual(len(nm), 9999)

    def test_folder_size_bad_folder(self):
        self.assertRaises(exc.FolderNotFound, utils.folder_size, "/doesnt_exist")

    def test_folder_size_no_ignore(self):
        with utils.SelfDeletingTempDirectory() as tmpdir:
            # write 10 files of 100 bytes each
            content = "x" * 100
            for idx in xrange(10):
                pth = os.path.join(tmpdir, "test%s" % idx)
                with file(pth, "w") as ff:
                    ff.write(content)
            fsize = utils.folder_size(tmpdir)
        self.assertEqual(fsize, 1000)

    def test_folder_size_ignore_string(self):
        with utils.SelfDeletingTempDirectory() as tmpdir:
            # write 10 files of 100 bytes each
            content = "x" * 100
            for idx in xrange(10):
                pth = os.path.join(tmpdir, "test%s" % idx)
                with file(pth, "w") as ff:
                    ff.write(content)
            # ignore one file
            fsize = utils.folder_size(tmpdir, ignore="*7")
        self.assertEqual(fsize, 900)

    def test_folder_size_ignore_list(self):
        with utils.SelfDeletingTempDirectory() as tmpdir:
            # write 10 files of 100 bytes each
            content = "x" * 100
            for idx in xrange(10):
                pth = os.path.join(tmpdir, "test%s" % idx)
                with file(pth, "w") as ff:
                    ff.write(content)
            # ignore odd files
            ignore = ["*1", "*3", "*5", "*7", "*9"]
            fsize = utils.folder_size(tmpdir, ignore=ignore)
        self.assertEqual(fsize, 500)

    def test_add_method(self):
        def fake_method(self):
            pass
        obj = fakes.FakeEntity()
        utils.add_method(obj, fake_method, "fake_name")
        self.assertTrue(hasattr(obj, "fake_name"))
        self.assertTrue(callable(obj.fake_name))

    def test_env(self):
        args = ("foo", "bar")
        ret = utils.env(*args)
        self.assertFalse(ret)
        os.environ["bar"] = "test"
        ret = utils.env(*args)
        self.assertEqual(ret, "test")

    def test_unauthenticated(self):
        def dummy(): pass
        utils.unauthenticated(dummy)
        self.assertTrue(hasattr(dummy, "unauthenticated"))

    def test_isunauthenticated(self):
        def dummy(): pass
        self.assertFalse(utils.isunauthenticated(dummy))
        utils.unauthenticated(dummy)
        self.assertTrue(utils.isunauthenticated(dummy))

    def test_safe_issubclass_good(self):
        ret = utils.safe_issubclass(fakes.FakeIdentity, fakes.Identity)
        self.assertTrue(ret)

    def test_safe_issubclass_bad(self):
        fake = fakes.FakeEntity()
        ret = utils.safe_issubclass(fake, None)
        self.assertFalse(ret)

    def test_slugify(self):
        test = "SAMPLE test_with-hyphen"
        expected = u"sample-test_with-hyphen"
        ret = utils.slugify(test)
        self.assertEqual(ret, expected)

    def test_wait_until(self):
        status_obj = fakes.FakeStatusChanger()
        self.assertRaises(exc.NoReloadError, utils.wait_until, status_obj, "status", "available")
        status_obj.manager = fakes.FakeManager()
        status_obj.manager.get = Mock(return_value=status_obj)

        ret = utils.wait_until(status_obj, "status", "ready", interval=0.1)
        self.assertTrue(ret)
        self.assertEqual(status_obj.status, "ready")
        ret = utils.wait_until(status_obj, "status", "fake", interval=0.1, attempts=2)

    def test_import_class(self):
        cls_string = "tests.unit.fakes.FakeManager"
        ret = utils.import_class(cls_string)
        self.assertTrue(ret is fakes.FakeManager)



if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2012 Rackspace

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import pyrax

creds_file = os.path.expanduser("~/.rackspace_cloud_credentials")
pyrax.set_credential_file(creds_file)
cbs = pyrax.cloud_blockstorage

# This assumes that you have are deleting the volumes named 'my_fast_volume'
# and 'my_standard_volume' that were created in create_volume.py.
for nm in ("my_fast_volume", "my_standard_volume"):
    try:
        vol = cbs.findall(name=nm)[0]
    except IndexError:
        print "There is no volume named '%s'. Skipping..." % nm
        vol = None
    if vol:
        print "Deleting", vol
        vol.delete()
print
print "Done."
print



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from mock import patch
from mock import MagicMock as Mock

from pyrax import CloudDatabaseClient
from pyrax import CloudDatabaseDatabase
from pyrax import CloudDatabaseFlavor
from pyrax import CloudDatabaseInstance
from pyrax import CloudDatabaseUser
from pyrax.cloud_databases import assure_instance
import pyrax.exceptions as exc
import pyrax.utils as utils

from tests.unit import fakes

example_uri = "http://example.com"


class CloudDatabasesTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(CloudDatabasesTest, self).__init__(*args, **kwargs)

    def setUp(self):
        self.instance = fakes.FakeDatabaseInstance()
        self.client = fakes.FakeDatabaseClient()

    def tearDown(self):
        pass

    def test_assure_instance(self):
        class TestClient(object):
            _manager = fakes.FakeManager()
            
            @assure_instance
            def test_method(self, instance):
                return instance

        client = TestClient()
        client._manager.get = Mock(return_value=self.instance)
        # Pass the instance
        ret = client.test_method(self.instance)
        self.assertTrue(ret is self.instance)
        # Pass the ID
        ret = client.test_method(self.instance.id)
        self.assertTrue(ret is self.instance)

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_instantiate_instance(self):
        inst = CloudDatabaseInstance(fakes.FakeManager(), {"id": 42})
        self.assertTrue(isinstance(inst, CloudDatabaseInstance))

    def test_list_databases(self):
        inst = self.instance
        sav = inst._database_manager.list
        inst._database_manager.list = Mock()
        inst.list_databases()
        inst._database_manager.list.assert_called_once_with()
        inst._database_manager.list = sav

    def test_list_users(self):
        inst = self.instance
        sav = inst._user_manager.list
        inst._user_manager.list = Mock()
        inst.list_users()
        inst._user_manager.list.assert_called_once_with()
        inst._user_manager.list = sav

    def test_get_database(self):
        inst = self.instance
        sav = inst.list_databases
        db1 = fakes.FakeEntity()
        db1.name = "a"
        db2 = fakes.FakeEntity()
        db2.name = "b"
        inst.list_databases = Mock(return_value=[db1, db2])
        ret = inst.get_database("a")
        self.assertEqual(ret, db1)
        inst.list_databases = sav

    def test_get_database_bad(self):
        inst = self.instance
        sav = inst.list_databases
        db1 = fakes.FakeEntity()
        db1.name = "a"
        db2 = fakes.FakeEntity()
        db2.name = "b"
        inst.list_databases = Mock(return_value=[db1, db2])
        self.assertRaises(exc.NoSuchDatabase, inst.get_database, "z")
        inst.list_databases = sav

    def test_create_database(self):
        inst = self.instance
        sav = inst._database_manager.create
        inst._database_manager.create = Mock()
        db = inst.create_database(name="test")
        inst._database_manager.create.assert_called_once_with(name="test",
                character_set="utf8", collate="utf8_general_ci", return_none=True)
        inst._database_manager.create = sav

    def test_create_user(self):
        inst = self.instance
        sav = inst._user_manager.create
        inst._user_manager.create = Mock()
        inst.create_user(name="test", password="testpw", database_names="testdb")
        inst._user_manager.create.assert_called_once_with(name="test", password="testpw",
        database_names=["testdb"], return_none=True)
        inst._user_manager.create = sav

    def test_delete_database(self):
        inst = self.instance
        sav = inst._database_manager.delete
        inst._database_manager.delete = Mock()
        inst.delete_database("dbname")
        inst._database_manager.delete.assert_called_once_with("dbname")
        inst._database_manager.delete = sav

    def test_delete_user(self):
        inst = self.instance
        sav = inst._user_manager.delete
        inst._user_manager.delete = Mock()
        inst.delete_user("username")
        inst._user_manager.delete.assert_called_once_with("username")
        inst._user_manager.delete = sav

    def test_enable_root_user(self):
        inst = self.instance
        pw = utils.random_name()
        fake_body = {"user": {"password": pw}}
        inst.manager.api.method_post = Mock(return_value=(None, fake_body))
        ret = inst.enable_root_user()
        call_uri = "/instances/%s/root" % inst.id
        inst.manager.api.method_post.assert_called_once_with(call_uri)
        self.assertEqual(ret, pw)

    def test_root_user_status(self):
        inst = self.instance
        fake_body = {"rootEnabled": True}
        inst.manager.api.method_get = Mock(return_value=(None, fake_body))
        ret = inst.root_user_status()
        call_uri = "/instances/%s/root" % inst.id
        inst.manager.api.method_get.assert_called_once_with(call_uri)
        self.assertTrue(ret)

    def test_restart(self):
        inst = self.instance
        inst.manager.action = Mock()
        ret = inst.restart()
        inst.manager.action.assert_called_once_with(inst, "restart")

    def test_resize(self):
        inst = self.instance
        flavor_ref = utils.random_name()
        inst.manager.api._get_flavor_ref = Mock(return_value=flavor_ref)
        fake_body = {"flavorRef": flavor_ref}
        inst.manager.action = Mock()
        ret = inst.resize(42)
        call_uri = "/instances/%s/action" % inst.id
        inst.manager.action.assert_called_once_with(inst, "resize", body=fake_body)

    def test_resize_volume_too_small(self):
        inst = self.instance
        inst.volume.get = Mock(return_value=2)
        self.assertRaises(exc.InvalidVolumeResize, inst.resize_volume, 1)

    def test_resize_volume(self):
        inst = self.instance
        inst.volume.get = Mock(return_value=1)
        fake_body = {"volume": {"size": 2}}
        inst.manager.action = Mock()
        ret = inst.resize_volume(2)
        inst.manager.action.assert_called_once_with(inst, "resize", body=fake_body)

    def test_get_flavor_property(self):
        inst = self.instance
        inst._loaded = True
        flavor = inst.flavor
        self.assertTrue(isinstance(flavor, CloudDatabaseFlavor))

    def test_set_flavor_property_dict(self):
        inst = self.instance
        inst._loaded = True
        inst.flavor = {"name": "test"}
        self.assertTrue(isinstance(inst.flavor, CloudDatabaseFlavor))

    def test_set_flavor_property_instance(self):
        inst = self.instance
        inst._loaded = True
        flavor = CloudDatabaseFlavor(inst.manager, {"name": "test"})
        inst.flavor = flavor
        self.assertTrue(isinstance(inst.flavor, CloudDatabaseFlavor))

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_list_databases_for_instance(self):
        clt = self.client
        inst = self.instance
        sav = inst.list_databases
        inst.list_databases = Mock(return_value=["db"])
        ret = clt.list_databases(inst)
        self.assertEqual(ret, ["db"])
        inst.list_databases.assert_called_once_with()
        inst.list_databases = sav

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_create_database_for_instance(self):
        clt = self.client
        inst = self.instance
        sav = inst.create_database
        inst.create_database = Mock(return_value=["db"])
        nm = utils.random_name()
        ret = clt.create_database(inst, nm)
        self.assertEqual(ret, ["db"])
        inst.create_database.assert_called_once_with(nm,
                character_set=None, collate=None)
        inst.create_database = sav

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_delete_database_for_instance(self):
        clt = self.client
        inst = self.instance
        sav = inst.delete_database
        inst.delete_database = Mock()
        nm = utils.random_name()
        clt.delete_database(inst, nm)
        inst.delete_database.assert_called_once_with(nm)
        inst.delete_database = sav

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_list_users_for_instance(self):
        clt = self.client
        inst = self.instance
        sav = inst.list_users
        inst.list_users = Mock(return_value=["user"])
        ret = clt.list_users(inst)
        self.assertEqual(ret, ["user"])
        inst.list_users.assert_called_once_with()
        inst.list_users = sav

    def test_create_user_for_instance(self):
        clt = self.client
        inst = self.instance
        sav = inst.create_user
        inst.create_user = Mock()
        nm = utils.random_name()
        pw = utils.random_name()
        ret = clt.create_user(inst, nm, pw, ["db"])
        inst.create_user.assert_called_once_with(name=nm, password=pw,
                database_names=["db"])
        inst.create_user = sav

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_delete_user_for_instance(self):
        clt = self.client
        inst = self.instance
        sav = inst.delete_user
        inst.delete_user = Mock()
        nm = utils.random_name()
        clt.delete_user(inst, nm)
        inst.delete_user.assert_called_once_with(nm)
        inst.delete_user = sav

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_enable_root_user_for_instance(self):
        clt = self.client
        inst = self.instance
        sav = inst.enable_root_user
        inst.enable_root_user = Mock()
        clt.enable_root_user(inst)
        inst.enable_root_user.assert_called_once_with()
        inst.enable_root_user = sav

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_root_user_status_for_instance(self):
        clt = self.client
        inst = self.instance
        sav = inst.root_user_status
        inst.root_user_status = Mock()
        clt.root_user_status(inst)
        inst.root_user_status.assert_called_once_with()
        inst.root_user_status = sav

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_resize_for_instance(self):
        clt = self.client
        inst = self.instance
        sav = inst.resize
        inst.resize = Mock()
        clt.resize(inst, "flavor")
        inst.resize.assert_called_once_with("flavor")
        inst.resize = sav

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_list_flavors(self):
        clt = self.client
        clt._flavor_manager.list = Mock()
        clt.list_flavors()
        clt._flavor_manager.list.assert_called_once_with()

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_get_flavor(self):
        clt = self.client
        clt._flavor_manager.get = Mock()
        clt.get_flavor("flavorid")
        clt._flavor_manager.get.assert_called_once_with("flavorid")

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_get_flavor_ref_for_obj(self):
        clt = self.client
        info = {"id": 1,
                "name": "test_flavor",
                "ram": 42,
                "links": [{
                "href": example_uri,
                "rel": "self"}]}
        flavor_obj = CloudDatabaseFlavor(clt._manager, info)
        ret = clt._get_flavor_ref(flavor_obj)
        self.assertEqual(ret, example_uri)

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_get_flavor_ref_for_id(self):
        clt = self.client
        info = {"id": 1,
                "name": "test_flavor",
                "ram": 42,
                "links": [{
                "href": example_uri,
                "rel": "self"}]}
        flavor_obj = CloudDatabaseFlavor(clt._manager, info)
        sav = clt.get_flavor
        clt.get_flavor = Mock(return_value=flavor_obj)
        ret = clt._get_flavor_ref(1)
        self.assertEqual(ret, example_uri)
        clt.get_flavor = sav

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_get_flavor_ref_for_name(self):
        clt = self.client
        info = {"id": 1,
                "name": "test_flavor",
                "ram": 42,
                "links": [{
                "href": example_uri,
                "rel": "self"}]}
        flavor_obj = CloudDatabaseFlavor(clt._manager, info)
        sav_get = clt.get_flavor
        sav_list = clt.list_flavors
        clt.get_flavor = Mock(side_effect=exc.NotFound(""))
        clt.list_flavors = Mock(return_value=[flavor_obj])
        ret = clt._get_flavor_ref("test_flavor")
        self.assertEqual(ret, example_uri)
        clt.get_flavor = sav_get
        clt.list_flavors = sav_list

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_get_flavor_ref_for_name(self):
        clt = self.client
        info = {"id": 1,
                "name": "test_flavor",
                "ram": 42,
                "links": [{
                "href": example_uri,
                "rel": "self"}]}
        flavor_obj = CloudDatabaseFlavor(clt._manager, info)
        sav_get = clt.get_flavor
        sav_list = clt.list_flavors
        clt.get_flavor = Mock(side_effect=exc.NotFound(""))
        clt.list_flavors = Mock(return_value=[flavor_obj])
        ret = clt._get_flavor_ref(42)
        self.assertEqual(ret, example_uri)
        clt.get_flavor = sav_get
        clt.list_flavors = sav_list

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_get_flavor_ref_not_found(self):
        clt = self.client
        info = {"id": 1,
                "name": "test_flavor",
                "ram": 42,
                "links": [{
                "href": example_uri,
                "rel": "self"}]}
        flavor_obj = CloudDatabaseFlavor(clt._manager, info)
        sav_get = clt.get_flavor
        sav_list = clt.list_flavors
        clt.get_flavor = Mock(side_effect=exc.NotFound(""))
        clt.list_flavors = Mock(return_value=[flavor_obj])
        self.assertRaises(exc.FlavorNotFound, clt._get_flavor_ref, "nonsense")
        clt.get_flavor = sav_get
        clt.list_flavors = sav_list

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_create_body_db(self):
        clt = self.client
        nm = utils.random_name()
        ret = clt._create_body(nm, character_set="CS", collate="CO")
        expected = {"databases": [
                {"name": nm,
                "character_set": "CS",
                "collate": "CO"}]}
        self.assertEqual(ret, expected)

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_create_body_user(self):
        clt = self.client
        nm = utils.random_name()
        pw = utils.random_name()
        ret = clt._create_body(nm, password=pw, database_names=[])
        expected = {"users": [
                {"name": nm,
                "password": pw,
                "databases": []}]}
        self.assertEqual(ret, expected)

    @patch("pyrax.manager.BaseManager", new=fakes.FakeManager)
    def test_create_body_flavor(self):
        clt = self.client
        nm = utils.random_name()
        sav = clt._get_flavor_ref
        clt._get_flavor_ref = Mock(return_value=example_uri)
        ret = clt._create_body(nm)
        expected = {"instance": {
                "name": nm,
                "flavorRef": example_uri,
                "volume": {"size": 1},
                "databases": [],
                "users": []}}
        self.assertEqual(ret, expected)
        clt._get_flavor_ref = sav


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2012 Rackspace

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import time

import pyrax


creds_file = os.path.expanduser("~/.rackspace_cloud_credentials")
pyrax.set_credential_file(creds_file)
clb = pyrax.cloud_loadbalancers

lb = clb.list()[0]
print
print "Load Balancer:", lb
print
print "Current nodes:", lb.nodes

# You may have to adjust the address of the node to something on
# the same internal network as your load balancer.
new_node = clb.Node(address="10.177.1.2", port=80, condition="ENABLED")
lb.add_nodes([new_node])
pyrax.utils.wait_until(lb, "status", "ACTIVE", interval=1, attempts=30, verbose=True)

print
print "After adding node:", lb.nodes

# Now remove that node. Note that we can't use the original node instance,
# as it was created independently, and doesn't have the link to its load
# balancer. Instead, we'll get the last node from the load balancer.
added_node = [node for node in lb.nodes
        if node.address == new_node.address][0]
print
print "Added Node:", added_node
added_node.delete()
pyrax.utils.wait_until(lb, "status", "ACTIVE", interval=1, attempts=30, verbose=True)
print
print "After removing node:", lb.nodes



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2012 Rackspace

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

class PyraxException(Exception):
    pass

class AccessListIDNotFound(PyraxException):
    pass

class AuthenticationFailed(PyraxException):
    pass

class AuthorizationFailure(PyraxException):
    pass

class AuthSystemNotFound(PyraxException):
    pass

class CDNFailed(PyraxException):
    pass

class EndpointNotFound(PyraxException):
    pass

class FlavorNotFound(PyraxException):
    pass

class FileNotFound(PyraxException):
    pass

class FolderNotFound(PyraxException):
    pass

class InvalidCDNMetadata(PyraxException):
    pass

class InvalidConfigurationFile(PyraxException):
    pass

class InvalidCredentialFile(PyraxException):
    pass

class InvalidDateTimeString(PyraxException):
    pass

class InvalidNodeCondition(PyraxException):
    pass

class InvalidNodeParameters(PyraxException):
    pass

class InvalidSessionPersistenceType(PyraxException):
    pass

class InvalidSize(PyraxException):
    pass

class InvalidUploadID(PyraxException):
    pass

class InvalidVirtualIPType(PyraxException):
    pass

class InvalidVirtualIPVersion(PyraxException):
    pass

class InvalidVolumeResize(PyraxException):
    pass

class MissingHealthMonitorSettings(PyraxException):
    pass

class MissingName(PyraxException):
    pass

class NoReloadError(PyraxException):
    pass

class NoSSLTerminationConfiguration(PyraxException):
    pass

class NoSuchContainer(PyraxException):
    pass

class NoSuchDatabase(PyraxException):
    pass

class NoSuchDatabaseUser(PyraxException):
    pass

class NoSuchObject(PyraxException):
    pass

class NotAuthenticated(PyraxException):
    pass

class NotCDNEnabled(PyraxException):
    pass

class NoTokenLookupException(PyraxException):
    pass

class ProtocolMismatch(PyraxException):
    pass

class SnapshotNotAvailable(PyraxException):
    pass

class UnattachedNode(PyraxException):
    pass

class UnattachedVirtualIP(PyraxException):
    pass

class Unauthorized(PyraxException):
    pass

class UploadFailed(PyraxException):
    pass

class VolumeNotAvailable(PyraxException):
    pass


class AmbiguousEndpoints(PyraxException):
    """Found more than one matching endpoint in Service Catalog."""
    def __init__(self, endpoints=None):
        self.endpoints = endpoints

    def __str__(self):
        return "AmbiguousEndpoints: %s" % repr(self.endpoints)


class ClientException(PyraxException):
    """
    The base exception class for all exceptions this library raises.
    """
    def __init__(self, code, message=None, details=None, request_id=None):
        self.code = code
        self.message = message or self.__class__.message
        self.details = details
        self.request_id = request_id

    def __str__(self):
        formatted_string = "%s (HTTP %s)" % (self.message, self.code)
        if self.request_id:
            formatted_string += " (Request-ID: %s)" % self.request_id

        return formatted_string

class BadRequest(ClientException):
    """
    HTTP 400 - Bad request: you sent some malformed data.
    """
    http_status = 400
    message = "Bad request"


class Unauthorized(ClientException):
    """
    HTTP 401 - Unauthorized: bad credentials.
    """
    http_status = 401
    message = "Unauthorized"


class Forbidden(ClientException):
    """
    HTTP 403 - Forbidden: your credentials don't give you access to this
    resource.
    """
    http_status = 403
    message = "Forbidden"


class NotFound(ClientException):
    """
    HTTP 404 - Not found
    """
    http_status = 404
    message = "Not found"


class NoUniqueMatch(ClientException):
    """
    HTTP 400 - Bad Request
    """
    http_status = 400
    message = "Not Unique"


class OverLimit(ClientException):
    """
    HTTP 413 - Over limit: you're over the API limits for this time period.
    """
    http_status = 413
    message = "Over limit"


# NotImplemented is a python keyword.
class HTTPNotImplemented(ClientException):
    """
    HTTP 501 - Not Implemented: the server does not support this operation.
    """
    http_status = 501
    message = "Not Implemented"



# In Python 2.4 Exception is old-style and thus doesn't have a __subclasses__()
# so we can do this:
#     _code_map = dict((c.http_status, c)
#                      for c in ClientException.__subclasses__())
#
# Instead, we have to hardcode it:
_code_map = dict((c.http_status, c) for c in [BadRequest, Unauthorized,
        Forbidden, NotFound, OverLimit, HTTPNotImplemented])


def from_response(response, body):
    """
    Return an instance of a ClientException or subclass
    based on an httplib2 response.

    Usage::

        resp, body = http.request(...)
        if resp.status != 200:
            raise exception_from_response(resp, body)
    """
    cls = _code_map.get(response.status, ClientException)
    request_id = response.get("x-compute-request-id")
    if body:
        message = "n/a"
        details = "n/a"
        if hasattr(body, "keys"):
            error = body[body.keys()[0]]
            if isinstance(error, dict):
                message = error.get("message", None)
                details = error.get("details", None)
            else:
                message = error
                details = None
        return cls(code=response.status, message=message, details=details,
                   request_id=request_id)
    else:
        return cls(code=response.status, request_id=request_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2012 Rackspace

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import pyrax

creds_file = os.path.expanduser("~/.rackspace_cloud_credentials")
pyrax.set_credential_file(creds_file)
clb = pyrax.cloud_loadbalancers

lb = clb.list()[0]
orig_meta = lb.get_metadata()
print "Initial metadata:", orig_meta
lb.set_metadata({"a": "one", "b": "two", "c": "three"})
print "New metadata:", lb.get_metadata()
lb.update_metadata({"d": "four"})
print "Updated metadata:", lb.get_metadata()
lb.set_metadata({"e": "five"})
print "After set_metadata:", lb.get_metadata()
lb.delete_metadata()
print "After delete_metadata:", lb.get_metadata()
if orig_meta:
    lb.set_metadata(orig_meta)
    print "After restoring original metadata:", lb.get_metadata()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import datetime
import httplib2
import json
import os
import pkg_resources
import unittest
import urllib2

from mock import patch
from mock import MagicMock as Mock

import pyrax.utils as utils
import pyrax.exceptions as exc
from pyrax import client

from tests.unit import fakes


class ClientTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(ClientTest, self).__init__(*args, **kwargs)

    def setUp(self):
        save_conf = client.BaseClient._configure_manager
        client.BaseClient._configure_manager = Mock()
        self.client = client.BaseClient(user="fake", password="fake")
        client.BaseClient._configure_manager = save_conf
        self.client._manager = fakes.FakeManager()

    def tearDown(self):
        self.client = None

    def test_get_auth_system_url(self):
        save_priep = pkg_resources.iter_entry_points
        pkg_resources.iter_entry_points = Mock()
        pkg_resources.iter_entry_points.return_value = fakes.fakeEntryPoints
        ret = client.get_auth_system_url("b")
        self.assertEqual(ret, "b")
        self.assertRaises(exc.AuthSystemNotFound, client.get_auth_system_url, "z")
        pkg_resources.iter_entry_points = save_priep

    def test_base_client(self):
        user = "fakeuser"
        password = "fakepassword"
        tenant_id = "faketenantid"
        auth_url = "fakeauthurl"
        region_name = "fakeregion"
        endpoint_type = "fakeenpointtype"
        management_url = "fakemanagementurl"
        auth_token = "fakeauthtoken"
        service_type = "fakeservicetype"
        service_name = "fakeservicename"
        timings = "faketimings"
        no_cache = "fakenocache"
        http_log_debug = "fakehttplogdebug"
        timeout = "faketimeout"
        auth_system = "fakeauthsystem"

        save_conf = client.BaseClient._configure_manager
        client.BaseClient._configure_manager = Mock()
        bc = client.BaseClient(user=user, password=password,
                tenant_id=tenant_id, auth_url=auth_url,
                region_name=region_name, endpoint_type=endpoint_type,
                management_url=management_url, auth_token=auth_token,
                service_type=service_type, service_name=service_name,
                timings=timings, no_cache=no_cache, http_log_debug=http_log_debug,
                timeout=timeout, auth_system=auth_system)

        self.assertEqual(bc.user, user)
        self.assertEqual(bc.password, password)
        self.assertEqual(bc.tenant_id, tenant_id)
        self.assertEqual(bc.auth_url, auth_url)
        self.assertEqual(bc.region_name, region_name)
        self.assertEqual(bc.endpoint_type, endpoint_type)
        self.assertEqual(bc.management_url, management_url)
        self.assertEqual(bc.auth_token, auth_token)
        self.assertEqual(bc.service_type, service_type)
        self.assertEqual(bc.service_name, service_name)
        self.assertEqual(bc.timings, timings)
        self.assertEqual(bc.no_cache, no_cache)
        self.assertEqual(bc.http_log_debug, http_log_debug)
        self.assertEqual(bc.timeout, timeout)
        self.assertEqual(bc.auth_system, auth_system)
        client.BaseClient._configure_manager = save_conf

    def test_configure_manager(self):
        self.assertRaises(NotImplementedError, self.client._configure_manager)

    def test_list(self):
        mgr = self.client._manager
        sav = mgr.list
        mgr.list = Mock()
        self.client.list()
        mgr.list.assert_called_once_with()
        mgr.list = sav

    def test_get(self):
        mgr = self.client._manager
        sav = mgr.get
        mgr.get = Mock()
        self.client.get("val")
        mgr.get.assert_called_once_with("val")
        mgr.get = sav

    def test_delete(self):
        mgr = self.client._manager
        sav = mgr.delete
        mgr.delete = Mock()
        self.client.delete("val")
        mgr.delete.assert_called_once_with("val")
        mgr.delete = sav

    def test_create(self):
        mgr = self.client._manager
        sav = mgr.create
        mgr.create = Mock()
        self.client.create("val")
        mgr.create.assert_called_once_with("val")
        mgr.create = sav

    def test_unauthenticate(self):
        clt = self.client
        clt.unauthenticate()
        self.assertIsNone(clt.management_url)
        self.assertIsNone(clt.auth_token)
        self.assertFalse(clt.used_keyring)

    def test_get_timings(self):
        clt = self.client
        clt.times = expected = [1, 2, 3]
        self.assertEqual(clt.get_timings(), expected)

    def test_reset_timings(self):
        clt = self.client
        clt.times = [1, 2, 3]
        clt.reset_timings()
        self.assertEqual(clt.get_timings(), [])

    def test_http_log_req(self):
        clt = self.client
        args = ("a", "b")
        kwargs = {"headers": {"c": "C"}}
        clt.http_log_debug = False
        self.assertIsNone(clt.http_log_req(args, kwargs))
        clt.http_log_debug = True
        sav = clt._logger.debug
        clt._logger.debug = Mock()
        clt.http_log_req(args, kwargs)
        clt._logger.debug.assert_called_once_with("\nREQ: curl -i a b -H 'c: C'\n")
        kwargs["body"] = "text"
        clt.http_log_req(args, kwargs)
        cargs, ckw = clt._logger.debug.call_args
        self.assertEqual(cargs, ("REQ BODY: text\n", ))
        clt._logger.debug = sav

    def test_http_log_resp(self):
        clt = self.client
        sav = clt._logger.debug
        clt._logger.debug = Mock()
        resp = "resp"
        body = "body"
        clt.http_log_debug = False
        clt.http_log_resp(resp, body)
        self.assertFalse(clt._logger.debug.called)
        clt.http_log_debug = True
        clt.http_log_resp(resp, body)
        self.assertTrue(clt._logger.debug.called)
        clt._logger.debug.assert_called_once_with("RESP:%s %s\n", "resp", "body")
        clt._logger.debug = sav

    def test_request_ok(self):
        clt = self.client
        clt.http_log_debug = False
        fakeresp = fakes.FakeResponse()
        fakeresp.status = 200
        body_content = {"one": 2, "three": 4}
        fakebody = json.dumps(body_content)
        sav = httplib2.Http.request
        httplib2.Http.request = Mock(return_value=(fakeresp, fakebody))
        resp, body = clt.request(body="text")
        self.assertTrue(isinstance(resp, fakes.FakeResponse))
        self.assertEqual(resp.status, 200)
        self.assertEqual(body, body_content)
        httplib2.Http.request = sav

    def test_request_400(self):
        clt = self.client
        clt.http_log_debug = False
        fakeresp = fakes.FakeResponse()
        fakeresp.status = 400
        body_content = {"one": 2, "three": 4}
        fakebody = json.dumps(body_content)
        sav = httplib2.Http.request
        httplib2.Http.request = Mock(return_value=(fakeresp, fakebody))
        savexc = exc.from_response
        exc.from_response = Mock(side_effect=fakes.FakeException)
        self.assertRaises(fakes.FakeException, clt.request)
        exc.from_response = savexc
        httplib2.Http.request = sav

    def test_request_no_json_resp(self):
        clt = self.client
        clt.http_log_debug = False
        fakeresp = fakes.FakeResponse()
        fakeresp.status = 400
        body_content = {"one": 2, "three": 4}
        fakebody = json.dumps(body_content)
        sav = httplib2.Http.request
        # Test non-json response
        fakebody = "{{{{{{"
        httplib2.Http.request = Mock(return_value=(fakeresp, fakebody))
        savexc = exc.from_response
        exc.from_response = Mock(side_effect=fakes.FakeException)
        self.assertRaises(fakes.FakeException, clt.request)
        exc.from_response = savexc
        httplib2.Http.request = sav

    def test_request_empty_body(self):
        clt = self.client
        clt.http_log_debug = False
        fakeresp = fakes.FakeResponse()
        fakeresp.status = 400
        body_content = {"one": 2, "three": 4}
        fakebody = json.dumps(body_content)
        sav = httplib2.Http.request
        fakebody = ""
        httplib2.Http.request = Mock(return_value=(fakeresp, fakebody))
        savexc = exc.from_response
        exc.from_response = Mock(side_effect=fakes.FakeException)
        self.assertRaises(fakes.FakeException, clt.request)
        exc.from_response.assert_called_once_with(fakeresp, None)
        exc.from_response = savexc
        httplib2.Http.request = sav

    def test_time_request(self):
        clt = self.client
        sav = clt.request
        clt.request = Mock()
        url = "http://example.com"
        method = "PUT"
        clt.request(url, method)
        clt.request.assert_called_once_with(url, method)
        clt.request = sav

    def test_cs_request_not_authed(self):
        clt = self.client
        sav_auth = clt.authenticate
        clt.authenticate = Mock()
        sav_req = clt.request
        clt.request = Mock(return_value=(1,1))
        url = "http://example.com"
        method = "PUT"
        clt.unauthenticate()
        clt.management_url = ""
        clt._cs_request(url, method)
        clt.authenticate.assert_called_once_with()
        clt.request = sav_req
        clt.authenticate = sav_auth

    def test_cs_request_auth_failed(self):
        clt = self.client
        sav_auth = clt.authenticate
        clt.authenticate = Mock()
        sav_req = clt.request
        clt.request = Mock(return_value=(1,1))
        url = "http://example.com"
        method = "PUT"
        clt.request = Mock(side_effect=exc.Unauthorized(""))
        clt.management_url = clt.auth_token = clt.tenant_id = "test"
        self.assertRaises(exc.Unauthorized, clt._cs_request, url, method)
        clt.request = sav_req
        clt.authenticate = sav_auth

    def test_method_get(self):
        clt = self.client
        sav = clt._cs_request
        clt._cs_request = Mock()
        url = "http://example.com"
        clt.method_get(url)
        clt._cs_request.assert_called_once_with(url, "GET")
        clt._cs_request = sav

    def test_method_post(self):
        clt = self.client
        sav = clt._cs_request
        clt._cs_request = Mock()
        url = "http://example.com"
        clt.method_post(url)
        clt._cs_request.assert_called_once_with(url, "POST")
        clt._cs_request = sav

    def test_method_put(self):
        clt = self.client
        sav = clt._cs_request
        clt._cs_request = Mock()
        url = "http://example.com"
        clt.method_put(url)
        clt._cs_request.assert_called_once_with(url, "PUT")
        clt._cs_request = sav

    def test_method_delete(self):
        clt = self.client
        sav = clt._cs_request
        clt._cs_request = Mock()
        url = "http://example.com"
        clt.method_delete(url)
        clt._cs_request.assert_called_once_with(url, "DELETE")
        clt._cs_request = sav

    @patch('pyrax.service_catalog.ServiceCatalog', new=fakes.FakeServiceCatalog)
    def test_extract_service_catalog_ok(self):
        clt = self.client
        url = "http://example.com"
        resp = fakes.FakeResponse()
        body = ""
        clt._extract_service_catalog(url, resp, body)
        self.assertEqual(clt.management_url, "http://example.com")

    @patch('pyrax.service_catalog.ServiceCatalog', new=fakes.FakeServiceCatalog)
    def test_extract_service_catalog_ambiguous_ep(self):
        clt = self.client
        url = "http://example.com"
        resp = fakes.FakeResponse()
        body = ""
        clt.region_name = "ALL"
        self.assertRaises(exc.AmbiguousEndpoints, clt._extract_service_catalog, url, resp, body)

    @patch('pyrax.service_catalog.ServiceCatalog', new=fakes.FakeServiceCatalog)
    def test_extract_service_catalog_key_error(self):
        clt = self.client
        url = "http://example.com"
        resp = fakes.FakeResponse()
        body = ""
        clt.region_name = "KEY"
        self.assertRaises(exc.AuthorizationFailure, clt._extract_service_catalog, url, resp, body)

    @patch('pyrax.service_catalog.ServiceCatalog', new=fakes.FakeServiceCatalog)
    def test_extract_service_catalog_ep_not_found(self):
        clt = self.client
        url = "http://example.com"
        resp = fakes.FakeResponse()
        body = ""
        clt.region_name = "EP"
        self.assertRaises(exc.EndpointNotFound, clt._extract_service_catalog, url, resp, body)

    def test_extract_service_catalog_proxy_resp(self):
        clt = self.client
        url = "http://example.com"
        resp = fakes.FakeResponse()
        resp.status = 305
        resp["location"] = "TEST"
        body = ""
        ret = clt._extract_service_catalog(url, resp, body)
        self.assertEqual(ret, resp["location"])

    def test_extract_service_catalog_other_status(self):
        clt = self.client
        url = "http://example.com"
        resp = fakes.FakeResponse()
        resp.status = 666
        body = ""
        savexc = exc.from_response
        exc.from_response = Mock(side_effect=fakes.FakeException)
        self.assertRaises(fakes.FakeException, clt._extract_service_catalog, url, resp, body)
        exc.from_response.assert_called_once_with(resp, "")
        exc.from_response = savexc

    def test_fetch_endpoints_from_auth(self):
        clt = self.client
        sav_tr = clt._time_request
        clt._time_request = Mock(return_value=("resp", "body"))
        sav_ex = clt._extract_service_catalog
        clt._extract_service_catalog = Mock(return_value="TEST")
        url = "http://example.com"
        ret = clt._fetch_endpoints_from_auth(url)
        self.assertEqual(ret, "TEST")

    def test_authenticate_with_keyring(self):
        clt = self.client
        sav_has = client.has_keyring
        client.has_keyring = True
        sav_kr = client.keyring
        client.keyring = fakes.FakeKeyring()
        clt.no_cache = False
        clt.used_keyring = False
        clt.authenticate()
        self.assertEqual(clt.auth_token, "FAKE_TOKEN")
        self.assertEqual(clt.management_url, "FAKE_URL")
        client.has_keyring = sav_has
        client.keyring = sav_kr

    def test_authenticate_v2(self):
        clt = self.client
        sav_has = client.has_keyring
        client.has_keyring = False
        clt.auth_url = "http://example.com"
        clt.version = "v2.0"
        sav_v2 = clt._v2_auth
        clt._v2_auth = Mock(return_value=None)
        sav_fetch = clt._fetch_endpoints_from_auth
        clt._fetch_endpoints_from_auth = Mock()
        clt.auth_system = None
        clt.proxy_token = "TOKEN"
        clt.authenticate()
        self.assertEqual(clt.auth_token, "TOKEN")
        clt._fetch_endpoints_from_auth = sav_fetch
        clt._v2_auth = sav_v2
        client.has_keyring = sav_has

    def test_authenticate_plugin(self):
        clt = self.client
        sav_has = client.has_keyring
        client.has_keyring = False
        clt.auth_url = "http://example.com"
        clt.version = "v2.0"
        sav_plug = clt._plugin_auth
        clt._plugin_auth = Mock(return_value=None)
        sav_fetch = clt._fetch_endpoints_from_auth
        clt._fetch_endpoints_from_auth = Mock()
        clt.auth_system = "test"
        clt.authenticate()
        self.assertEqual(clt.auth_token, None)
        clt._fetch_endpoints_from_auth = sav_fetch
        clt._plugin_auth = sav_plug
        client.has_keyring = sav_has

    def test_authenticate_v1(self):
        clt = self.client
        sav_has = client.has_keyring
        client.has_keyring = False
        clt.auth_url = "http://example.com"
        clt.version = "v1.1"
        sav_v1 = clt._v1_auth
        clt._v1_auth = Mock(return_value=None)
        sav_fetch = clt._fetch_endpoints_from_auth
        clt._fetch_endpoints_from_auth = Mock()
        clt.authenticate()
        self.assertEqual(clt.auth_token, None)
        clt._fetch_endpoints_from_auth = sav_fetch
        clt._v1_auth = sav_v1
        client.has_keyring = sav_has

    def test_authenticate_v1_fail(self):
        clt = self.client
        sav_has = client.has_keyring
        client.has_keyring = False
        clt.auth_url = "http://example.com"
        clt.version = "v1.1"
        sav_v1 = clt._v1_auth
        sav_v2 = clt._v2_auth
        clt._v1_auth = Mock(side_effect=exc.AuthorizationFailure)
        clt._v2_auth = Mock()
        sav_fetch = clt._fetch_endpoints_from_auth
        clt._fetch_endpoints_from_auth = Mock()
        clt.authenticate()
        self.assertEqual(clt.auth_token, None)
        clt._fetch_endpoints_from_auth = sav_fetch
        clt._v1_auth = sav_v1
        clt._v2_auth = sav_v2
        client.has_keyring = sav_has

    def test_authenticate_store_keyring(self):
        clt = self.client
        sav_has = client.has_keyring
        client.has_keyring = True
        sav_kr = client.keyring
        client.keyring = fakes.FakeKeyring()
        client.keyring.get_password = Mock(side_effect=Exception)
        sav_no = clt.no_cache
        clt.no_cache = False
        clt.auth_url = "http://example.com"
        clt.version = "v1.1"
        sav_v1 = clt._v1_auth
        clt._v1_auth = Mock(return_value=None)
        clt.authenticate()
        self.assertTrue(client.keyring.password_set)
        clt._v1_auth = sav_v1
        clt.no_cache = sav_no
        client.has_keyring = sav_has
        client.keyring = sav_kr

    def test_authenticate_store_keyring_exc(self):
        clt = self.client
        sav_has = client.has_keyring
        client.has_keyring = True
        sav_kr = client.keyring
        client.keyring = fakes.FakeKeyring()
        sav_spw = client.keyring.set_password
        client.keyring.set_password = Mock(side_effect=Exception)
        sav_no = clt.no_cache
        clt.no_cache = False
        clt.auth_url = "http://example.com"
        clt.version = "v1.1"
        sav_v1 = clt._v1_auth
        clt._v1_auth = Mock(return_value=None)
        clt.authenticate()
        self.assertFalse(client.keyring.password_set)
        clt._v1_auth = sav_v1
        clt.no_cache = sav_no
        client.keyring.set_password = sav_spw
        client.has_keyring = sav_has
        client.keyring = sav_kr

    def test_v1_auth_with_token(self):
        clt = self.client
        clt.proxy_token = "TOKEN"
        self.assertRaises(exc.NoTokenLookupException, clt._v1_auth, "")

    def test_v1_auth_ok(self):
        clt = self.client
        clt.proxy_token = None
        sav_tr = clt._time_request
        fake_resp = fakes.FakeResponse()
        fake_resp.status = 200
        fake_resp["x-server-management-url"] = "http://example.com"
        fake_resp["x-auth-token"] = "TOKEN"
        fake_body = "body"
        fake_url = "http://identity.example.com"
        clt._time_request = Mock(return_value=(fake_resp, fake_body))
        clt._v1_auth(fake_url)
        self.assertEqual(clt.auth_url, fake_url)
        clt._time_request = sav_tr

    def test_v1_auth_fail(self):
        clt = self.client
        clt.proxy_token = None
        sav_tr = clt._time_request
        fake_resp = fakes.FakeResponse()
        fake_resp.status = 200
        fake_body = "body"
        fake_url = "http://identity.example.com"
        clt._time_request = Mock(return_value=(fake_resp, fake_body))
        self.assertRaises(exc.AuthorizationFailure, clt._v1_auth, fake_url)
        clt._time_request = sav_tr

    def test_v1_auth_305(self):
        clt = self.client
        clt.proxy_token = None
        sav_tr = clt._time_request
        fake_resp = fakes.FakeResponse()
        fake_resp.status = 305
        fake_resp["location"] = "TEST"
        fake_body = "body"
        fake_url = "http://identity.example.com"
        clt._time_request = Mock(return_value=(fake_resp, fake_body))
        ret = clt._v1_auth(fake_url)
        self.assertEqual(ret, "TEST")
        clt._time_request = sav_tr

    def test_v1_auth_other_status(self):
        clt = self.client
        clt.proxy_token = None
        sav_tr = clt._time_request
        fake_resp = fakes.FakeResponse()
        fake_resp.status = 666
        fake_body = "body"
        fake_url = "http://identity.example.com"
        clt._time_request = Mock(return_value=(fake_resp, fake_body))
        sav_fr = exc.from_response
        exc.from_response = Mock(side_effect=exc.Unauthorized(""))
        self.assertRaises(exc.Unauthorized, clt._v1_auth, fake_url)
        exc.from_response = sav_fr
        clt._time_request = sav_tr

    def test_plugin_auth(self):
        clt = self.client
        save_priep = pkg_resources.iter_entry_points
        pkg_resources.iter_entry_points = Mock()
        pkg_resources.iter_entry_points.return_value = fakes.fakeEntryPoints
        sav_au = clt.auth_system
        clt.auth_system = "b"
        ret = clt._plugin_auth("http://example.com")
        self.assertEqual(ret, "b")
        pkg_resources.iter_entry_points = save_priep

    def test_plugin_auth_not_found(self):
        clt = self.client
        save_priep = pkg_resources.iter_entry_points
        pkg_resources.iter_entry_points = Mock()
        pkg_resources.iter_entry_points.return_value = fakes.fakeEntryPoints
        sav_au = clt.auth_system
        clt.auth_system = "b"
        self.assertRaises(exc.AuthSystemNotFound, client.get_auth_system_url, "z")
        pkg_resources.iter_entry_points = save_priep

    def test_v2_auth(self):
        clt = self.client
        clt.tenant_id = "FAKE_TENANT"
        sav_au = clt._authenticate
        clt._authenticate = Mock()
        ret = clt._v2_auth("http://example.com")
        self.assertIsNone(ret)
        clt._authenticate = sav_au

    def test_authenticate(self):
        clt = self.client
        sav_tr = clt._time_request
        clt._time_request = Mock(return_value=("resp", "body"))
        sav_esc = clt._extract_service_catalog
        clt._extract_service_catalog = Mock(return_value=None)
        ret = clt._authenticate("url", "body")
        self.assertIsNone(ret)

    def test_project_id(self):
        clt = self.client
        clt.tenant_id = "FAKE"
        self.assertEqual(clt.projectid, "FAKE")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2012 Rackspace

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import os
import pyrax

creds_file = os.path.expanduser("~/.rackspace_cloud_credentials")
pyrax.set_credential_file(creds_file)
clb = pyrax.cloud_loadbalancers

# You can specify a type, address, IP version, and ID
# None are required; defaults are:
#    type: PUBLIC
#    address: None
#    ip_version: IPV4
#    id: None

vip = clb.VirtualIP()
print "Virtual IP (using defaults):", vip

vip = clb.VirtualIP(type="SERVICENET", address="1.2.3.4", ipVersion="IPV4", id=999)
print "Virtual IP (using supplied values):", vip



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2012 Rackspace

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import pyrax

creds_file = os.path.expanduser("~/.rackspace_cloud_credentials")
pyrax.set_credential_file(creds_file)
cf = pyrax.cloudfiles

cont_name = pyrax.utils.random_name()
cont = cf.create_container(cont_name)
print "Container:", cont
print "Before Making Public"
print "cdn_enabled", cont.cdn_enabled
print "cdn_ttl", cont.cdn_ttl
print "cdn_log_retention", cont.cdn_log_retention
print "cdn_uri", cont.cdn_uri
print "cdn_ssl_uri", cont.cdn_ssl_uri
print "cdn_streaming_uri", cont.cdn_streaming_uri

# Make it public
cont.make_public(ttl=1200)

# Now re-check the container's attributes
cont = cf.get_container(cont_name)
print
print "After Making Public"
print "cdn_enabled", cont.cdn_enabled
print "cdn_ttl", cont.cdn_ttl
print "cdn_log_retention", cont.cdn_log_retention
print "cdn_uri", cont.cdn_uri
print "cdn_ssl_uri", cont.cdn_ssl_uri
print "cdn_streaming_uri", cont.cdn_streaming_uri

# clean up
cont.delete()
